

c-----------------------------------------------------------------------------
c
c ROUTINE
c   Add_Semiss_Harvard_Inchem
c
c DESCRIPTION
c   This routine adds in the Harvard emissions, and is used just prior to
c   calling the chemistry solver.
c
c ARGUMENTS
c   iisoprene_num : index of isoprene in const
c   ico_num       : index of CO       in const
c   ipropene_num  : index of propene  in const
c   ino_num       : index of NO       in const
c   mw            : array of species' molecular weights (g/mol)
c   emiss_isop    : isoprene    emissions (kg/s)
c   emiss_monot   : monoterpene emissions (kg/s)
c   emiss_nox     : NOx         emissions (kg/s)
cc?
c   surf_emiss    : surface     emissions (units?)
c
c-----------------------------------------------------------------------------

      subroutine Add_Semiss_Harvard_Inchem
     &  (iisoprene_num, ico_num, ipropene_num, ino_num, mw,
     &   emiss_isop, emiss_monot, emiss_nox, conv_emiss, surf_emiss)
      
      use h_imp_emiss_constants
      use h_imp_dims
      implicit none
 
c#     include "imp_emiss_constants.h"
c#     include "imp_dims.h"
 

c     ----------------------
c     Argument declarations.
c     ----------------------

      integer :: iisoprene_num
      integer :: ico_num
      integer :: ipropene_num
      integer :: ino_num
      real*8  :: mw(num_species)
      real*8  :: emiss_isop (i1:i2, ju1:j2)
      real*8  :: emiss_monot(i1:i2, ju1:j2)
      real*8  :: emiss_nox  (i1:i2, ju1:j2)
      real*8  :: conv_emiss (i1:i2, ju1:j2)
      real*8  :: surf_emiss (i1:i2, ju1:j2, num_species)


c     ----------------
c     Begin execution.
c     ----------------

      if (pr_diag) then
        Write (6,*) 'Add_Semiss_Harvard_Inchem called by ', loc_proc
      end if


c     ----------------------
      if (iisoprene_num > 0) then
c     ----------------------

c       ----------------------------
c       Biogenic source of isoprene.
c       ----------------------------

        surf_emiss(:,:,iisoprene_num) =
     &    surf_emiss(:,:,iisoprene_num) +
     &    (emiss_isop(:,:) * conv_emiss(:,:) / mw(iisoprene_num))

      end if


c     ----------------
      if (ico_num > 0) then
c     ----------------

c       ----------------------------------------------
c       Biogenic source of CO from methanol oxidation,
c       scaled from isoprene.
c       ----------------------------------------------

        surf_emiss(:,:,ico_num) =
     &    surf_emiss(:,:,ico_num) +
     &    (emiss_isop(:,:) * conv_emiss(:,:) / mw(ico_num)) *
     &    ICO_FAC_ISOP


c       -------------------------------------------------
c       Biogenic source of CO from monoterpene oxidation.
c       -------------------------------------------------

        surf_emiss(:,:,ico_num) =
     &    surf_emiss(:,:,ico_num) +
     &    (emiss_monot(:,:) *  conv_emiss(:,:) / mw(ico_num)) *
     &    ICO_FAC_MONOT

      end if


c     ---------------------
      if (ipropene_num > 0) then
c     ---------------------

c       -------------------------------------------------
c       Biogenic source of propene, scaled from isoprene.
c       -------------------------------------------------

        surf_emiss(:,:,ipropene_num)  =
     &    surf_emiss(:,:,ipropene_num)  +
     &    (emiss_isop(:,:) * conv_emiss(:,:) / mw(iisoprene_num)) *
     &    BIOSCAL *
     &    (ATOMSC_PER_MOLECISOP / ATOMSC_PER_MOLECPRPE)

      end if


c     ----------------
      if (ino_num > 0) then
c     ----------------

c       -------------------
c       Soil source of NOx.
c       -------------------

        surf_emiss(:,:,ino_num) =
     &    surf_emiss(:,:,ino_num) +
     &    (emiss_nox(:,:) * conv_emiss(:,:) / mw(ino_num))

      end if


      return

      end
c ** End of modifications by Luis Olcese, UM
c ********************************************

