c=============================================================================
c
c $Id: hvread.F,v 1.1 2003/05/16 22:22:25 akinorii Exp $
c
c CODE DEVELOPER
c   Original code from Sandy Sillman, U of M.
c   IMPACT quadchem modifications:  Akinori Ito.
c
c FILE
c   hvread.F
c
c ORIGINAL FILE
c   jval1.f
c
c=============================================================================

c-----------------------------------------------------------------------------
c
c ROUTINE
c   hvread
c
c DESCRIPTION
c  THIS SUBROUTINE READS THE HV MATRIX ASSOCIATED WITH 'BHVRATES'.
c  THE MATRIX WAS GENERATED BY SILLMAN WITH
c  PROGRAM FOR PROCESSING TUV TABLE (TUVGRID)
c  Uses table generated by tuvtab2.f
c  with subroutines for use in testing program and photochemistry.
c
c ARGUMENTS
c   lsin : File ID for look-up table.
c
c-----------------------------------------------------------------------------

      subroutine Hvread(lsin,nhv,hvmat, hvmatb, jarray)

c Output matrix:  jarray(k,ig,jc) for
c   k=cases:  k=1 base case, k>1 adjustment factors
c             for cases identified in hvmatrix.
c   ig=matrix of altitudes and zenith angles
c   jc=species j-values

c Includes AUTOMATIC FILL-IN OF LAST ZENITH VALUE (nighttime)
c  AND BASE CASE IN ADJUSTMENT FACTORS.
c    (read from input table.  Base j-values are 
c      0 (nighttime) for base case j-values;  
c      1 for most adjustment factors, but
c      0 for cloud adjustment factors 11 and 12 
c        (which are relative to original cloud factor)

c INPUTS (none)
c  OUTPUTS
      integer :: nhv(22), lsin
      real*8 :: hvmat(22,40), hvmatb(22)
      real*8 :: jarray(80,510,56)
c      (note: current scale jarray (k,ig,jc):  k=1,56)

c  INTERNAL 
      integer :: i, j, k, m, n, jct, iwri
      integer ::  jaer, jcld,jalb, jtem
      real*8 :: x,y, hvmatz(22)
      character*4 aaa
c --------------------------------------------------
c TEST WRITE:  WRITE IF iwri=1
      iwri=0

c OPTION:  TOTAL NUMBER OF J-VALUES TO BE READ. (jct=4 test 26 trop 56 full)
c  CHANGE GRID:  TUVGRID1 (jct=4) vs TUVGRID2 (jct=26)
C    see also CHANGE GRID below.
c     jct=4
      jct=26

c INDEX OPTION:  jaer (=6), jcld (originally =9) jalb (=8)
      jaer=6
      jcld=9
      jalb=8
      jtem=13
c
c ZERO INPUT MATRIX
      do i=1,22
       nhv(i)=0
       hvmatb(i)=0.
       do j=1,40
        hvmat(i,j)=0.
       enddo
      enddo

c READ INPUT CONTROLS  (note read index, k, as well)
      do i=1,22
       read(lsin,11) aaa
       if(iwri.eq.1) write( 57 ,11) aaa
11     format(a4)
       read(lsin,12) k,j,x, y
       if(iwri.eq.1) write(57 ,12) k,j,x, y
12     format(2i5,2f10.4)
       if(j.eq.0) go to 101
       if(k.eq.0) go to 101
       nhv(k)=j
       hvmatb(k)=x
       hvmatz(k)=y
       read(lsin,13) (hvmat(k,n),n=1,nhv(k))
       if(iwri.eq.1) write( 57,13) (hvmat(k,n),n=1,nhv(k))
13     format (8f10.4)
100   continue
      enddo
101   continue

c ALT  ALTITUDE OPTION:  kPa or km
      do j=1,40
       hvmat(22,j)=hvmat(2,j)
c ALTITUDE OPTION;  FOR KM, UNCOMMENT THIS LINE.  FOR KPA, COMMENT OUT
       hvmat(2,j)=hvmat(21,j)
c END OPTION
      enddo

c SPECIAL TEMPERATURE TREATMENT:
c  INITIAL  READIN HAS nhv(jtem)=nhv(ialt);  and reads hvmat(jtem,j)
c   as standard temperature vs. altitude.
c  DATA READIN (jarray, below) reads  just one data  array,
c   representing parameter for std temp+10 degrees.
c  FOR THIS, RESET nhv(jtem)=2.
c  (hvmatb=hvmat(jtem,1); hvmatz=0.)

       if(jtem.gt.0) then
        if(nhv(jtem).gt.2)  nhv(jtem)=2
       endif

c SET INITIAL J-VALUES TO ZERO/ONE
c  J=0 FOR BASE CASE (k=1) AND CLOUD-ADJUSTMENT VALUES (k=11,12)
      do k=1,80
       do ig=1,510
        do jc=1,56
         jarray(k,ig,jc)=1.
         if(k.eq.1                      ) jarray(k,ig,jc)=0.
        enddo
       enddo
      enddo

c jarray(80,510,56)

c LOOP TO READ ARRAY BASE CASE (k=1) AND ADJUSTMENT CASES (k>1)
c  N REPRESENTS PARAMETER VALUE FROM J-MATRIX.    
c READ FROM ARRAY AND ENTER IMPLICIT VALUES OMITTED FROM ARRAY
       k=0
       do m=2,20
        nmax=1
        if(m.ge.3) nmax=nhv(m)
        if(iwri.eq.1) write(57,*) m,nmax,k
c      (Exit if nhv=0)
        if(nhv(m).le.0) go to 1001
c      (Note: for nhv=1, omit from jarray)       
        if(nhv(m).le.1) go to 1000
        do n=1,nmax
         k=k+1

         if(iwri.eq.1) write(57,*) m,n,k

c ENTER IMPLICIT VALUES FOR ARRAY (nighttime or base case adjustment factors)  
          do ig=1,510
           do jc=1,56 
            jarray(k,ig,jc)=hvmatz(m)
           enddo
          enddo

c READ ARRAY VALUES (skip for nighttime and base-case adjustment factors)
c                   (skip for cloud-above at alt>7)

c   altitude and zenith loop (skips nighttime zenith)
c     with control to skip base case adjustment factor
c     and to skip for ialt>7 for cloud-above (9, 11:  jcld, jcld+2)
c     NOTE CHANGE with added 1km layer:  ialt>8 skip

c   SKIPS BASE CASE ADJUSTMENT FACTOR IN ALL SUBSEQUENT LOOPS
c   BUT INCLUDES IT IN FIRST LOOP (m=2, k=2 was early error)
c         if(k.eq.2.or.(hvmat(m,n).ne.hvmatb(m))) then
          if(m.eq.2.or.(hvmat(m,n).ne.hvmatb(m))) then

c CASE HEADING
           read(lsin,11) aaa
           if(iwri.eq.1) write(57,*) k
           if(iwri.eq.1) write(57, 11) aaa
           do i=1,nhv(2)
c CHANGE GRID OPTION:   SKIP CONTROL - i.le.7 (TUVGRID1). i.le.8 FOR TUVGRID2
c  This  skips read for alt>8 for cloud-above or cl-alt-above.
c            if(i.le.7.or.(m.ne.9.and.m.ne.11)) then
             if(i.le.8.or.(m.ne.jcld.and.m.ne.jcld+2)) then

              read(lsin,11) aaa
              if(iwri.eq.1) write(57,*) k,i
              if(iwri.eq.1) write(57 ,11) aaa
              do iz=1,(nhv(1)-1)
               ig=iz+nhv(1)*(i-1)
               read(lsin,1002) (jarray(k,ig,jc),jc=1,jct)
               if(iwri.eq.1) write( 57 ,1002) (jarray(k,ig,jc),jc=1,4)
               if(ig.eq.1.and.iwri.eq.1) write(57,  * ) k
               if(k.eq.1.and.iz.le.2.and.i.le.2.and.iwri.eq.1)
     *          write(57,*) k,i,iz, ig
               if(k.eq.1.and.iz.le.2.and.i.le.2.and.iwri.eq.1)
     *         write( 57 ,1002) (jarray(k,ig,jc),jc=1,4)
 1002          format(8(1PE10.3))
              enddo
            endif
           enddo
          endif

c END LOOP TO READ
        enddo
1000   continue
       enddo
1001   continue

2000  return
      end
