
c=============================================================================
c
c $Id: imp_gen_io_utils.F,v 1.8 2003/04/25 22:17:12 jrt Exp $
c
c CODE DEVELOPER
c   John Tannahill, LLNL
c   jrt@llnl.gov
c
c FILE
c   imp_gen_io_utils.F
c
c ROUTINES
c   Asc_Open_Rd
c   Asc_Open_Wrt
c   Do_Err_Out
c   Does_File_Exist
c   Finish_File_Name
c   Make_Outfile_Name
c   Read_List
c   Check_Range_1d
c   Check_Range_2d
c   Check_Range_3d
c   Check_Range_4d
c   Check_Range_5d
c   Wrap_Master_2d
c   Wrap_Master_3du
c   Wrap_Master_3dv
c   Wrap_Master_4d
c
c=============================================================================


c-----------------------------------------------------------------------------
c
c ROUTINE
c   Asc_Open_Rd
c
c DESCRIPTION
c   This routine opens an ASCII file for reading and does some error
c   checking.
c
c ARGUMENTS
c   asc_lun     : logical unit for opened ASCII file
c   asc_filname : name of ASCII file to open for reading
c
c-----------------------------------------------------------------------------
 
      subroutine Asc_Open_Rd (asc_lun, asc_filname)
 
      implicit none
 
 
c     ----------------------
c     Argument declarations.
c     ----------------------
 
      integer :: asc_lun
      character (len=*) :: asc_filname
 
 
c     ----------------------
c     Variable declarations.
c     ----------------------
 
      character (len=75) :: err_msg

      integer :: ierr
      integer :: in1
 
 
c     ----------------
c     Begin execution.
c     ----------------
 
      ierr = 0

      call FI_Getlun (asc_lun, ierr)

      if (ierr /= 0) then

        in1 = Len_Trim (asc_filname)

        err_msg =
     &    'Open problem #1 in Asc_Open_Rd:  ' // asc_filname(1:in1)

        call Do_Err_Out (err_msg, .true., 1, ierr, 0, 0, 0.0d0, 0.0d0)

      end if


      ierr = 0

      Open (unit = asc_lun, file = asc_filname,
     &      status = 'old', iostat = ierr)

      if (ierr /= 0) then
 
        in1 = Len_Trim (asc_filname)

        err_msg =
     &    'Open problem #2 in Asc_Open_Rd:  ' // asc_filname(1:in1)

        call Do_Err_Out (err_msg, .true., 1, ierr, 0, 0, 0.0d0, 0.0d0)

      end if


      return
 
      end


c-----------------------------------------------------------------------------
c
c ROUTINE
c   Asc_Open_Wrt
c
c DESCRIPTION
c   This routine opens an ASCII file for writing and does some error
c   checking.
c
c ARGUMENTS
c   asc_lun     : logical unit for opened ASCII file
c   asc_filname : name of ASCII file to open for writing
c
c-----------------------------------------------------------------------------
 
      subroutine Asc_Open_Wrt (asc_lun, asc_filname)
 
      implicit none
 
 
c     ----------------------
c     Argument declarations.
c     ----------------------
 
      integer :: asc_lun
      character (len=*) :: asc_filname
 
 
c     ----------------------
c     Variable declarations.
c     ----------------------
 
      character (len=75) :: err_msg

      integer :: ierr
      integer :: in1
 
 
c     ----------------
c     Begin execution.
c     ----------------
 
      ierr = 0
 
      call FI_Getlun (asc_lun, ierr)
 
      if (ierr /= 0) then
 
        in1 = Len_Trim (asc_filname)
 
        err_msg =
     &    'Open problem #1 in Asc_Open_Wrt:  ' // asc_filname(1:in1)
 
        call Do_Err_Out (err_msg, .true., 1, ierr, 0, 0, 0.0d0, 0.0d0)
 
      end if


      ierr = 0
 
      Open (unit = asc_lun, file = asc_filname,
     &      status = 'unknown', iostat = ierr)
 
      if (ierr /= 0) then
 
        in1 = Len_Trim (asc_filname)
 
        err_msg =
     &    'Open problem #2 in Asc_Open_Wrt:  ' // asc_filname(1:in1)
 
        call Do_Err_Out (err_msg, .true., 1, ierr, 0, 0, 0.0d0, 0.0d0)
 
      end if
 

      return
 
      end


c-----------------------------------------------------------------------------
c
c ROUTINE
c   Do_Err_Out
c
c DESCRIPTION
c   This routine outputs error messages, and exits if requested.
c
c ARGUMENTS
c   err_msg       : error message to be printed out
c   err_do_stop   : do stop on error?
c   err_num_ints  : number of integers to be printed out (0, 1, or 2)
c   err_int1      : integer 1 to print out
c   err_int2      : integer 2 to print out
c   err_num_reals : number of reals to be printed out (0, 1, or 2)
c   err_real1     : real 1 to print out
c   err_real2     : real 2 to print out
c
c-----------------------------------------------------------------------------

      subroutine Do_Err_Out
     &  (err_msg, err_do_stop, err_num_ints, err_int1, err_int2,
     &   err_num_reals, err_real1, err_real2)
 
      implicit none

 
c     ----------------------
c     Argument declarations.
c     ----------------------

      character (len=*) :: err_msg
      logical :: err_do_stop
      integer :: err_num_ints
      integer :: err_int1
      integer :: err_int2
      integer :: err_num_reals
      real*8  :: err_real1
      real*8  :: err_real2


c     ----------------
c     Begin execution.
c     ----------------

      Write (6,*)
      Write (6,*)
     &  '!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'


      Write (6,*) '!! ' // Trim (err_msg)


      if (err_num_ints == 1) then

        Write (6,*) '!! ', err_int1

      else if (err_num_ints == 2) then

        Write (6,*) '!! ', err_int1, err_int2

      end if


      if (err_num_reals == 1) then

        Write (6,*) '!! ', err_real1

      else if (err_num_reals == 2) then

        Write (6,*) '!! ', err_real1, err_real2

      end if


      Write (6,*)
     &  '!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'
      Write (6,*)


c      if (err_do_stop) then
c        call Stopcode ("Code stopped from Do_Err_Out.")
c      end if


      return

      end


c-----------------------------------------------------------------------------
c
c ROUTINE
c   Does_File_Exist
c
c DESCRIPTION
c   This routine checks for the existence of a file.
c
c ARGUMENTS
c   fname : the name of the file to check
c
c-----------------------------------------------------------------------------
 
      function Does_File_Exist (fname)
 
      implicit none
 
 
c     ----------------------
c     Argument declarations.
c     ----------------------
 
      character (len=*) :: fname
 

c     ----------------------
c     Function declarations.
c     ----------------------
 
      logical :: Does_File_Exist


c     ----------------------
c     Variable declarations.
c     ----------------------
 
      logical :: does_exist
 
 
c     ----------------
c     Begin execution.
c     ----------------
 
      Inquire (file = fname, exist = does_exist)
 

      Does_File_Exist = does_exist


      return
 
      end


c-----------------------------------------------------------------------------
c
c ROUTINE
c   Finish_File_Name
c
c DESCRIPTION
c   This routine prepends the directory path to the file name if the file name
c   does not begin with a "/".
c
c ARGUMENTS
c   fname : the file name
c   dpath : the directory path
c
c-----------------------------------------------------------------------------

      subroutine Finish_File_Name (fname, dpath)

      implicit none
 

c     ----------------------
c     Argument declarations.
c     ----------------------

      character (len=*) :: fname
      character (len=*) :: dpath
 

c     ----------------------
c     Variable declarations.
c     ----------------------

      character (len=75)  :: err_msg
      character (len=128) :: tmp_string

      integer :: in1, in2


c     ----------------
c     Begin execution.
c     ----------------

      if (fname(1:1) /= '/') then

        in1 = Len_Trim (dpath)
        in2 = Len_Trim (fname)

        if (((in1 + in2) > 128) .or.
     &      (Len (fname) < 128)) then
          err_msg = 'Problem in Finish_File_Name.'
          call Do_Err_Out (err_msg, .true., 2, in1 + in2, Len (fname),
     &                     0, 0.0d0, 0.0d0)
        end if

        if ((in1 /= 0) .and. (in2 /= 0)) then
          tmp_string = dpath(1:in1) // '/' // fname(1:in2)
          fname      = tmp_string
        end if

      end if
 

      return

      end


c-----------------------------------------------------------------------------
c
c ROUTINE
c   Make_Outfile_Name
c
c DESCRIPTION
c   This routine uses the problem name and a suffix to create a name for an
c   output file.
c
c ARGUMENTS
c   fname  : the file name
c   suffix : the suffix to add after the file name is trimmed of blank
c            spaces
c
c-----------------------------------------------------------------------------

      subroutine Make_Outfile_Name (fname, suffix)

      use h_imp_gen_io
      implicit none

c#     include "imp_gen_io.h"
 

c     ----------------------
c     Argument declarations.
c     ----------------------

      character (len=*) :: fname
      character (len=*) :: suffix
 

c     ----------------------
c     Variable declarations.
c     ----------------------

      integer :: in1


c     ----------------
c     Begin execution.
c     ----------------

      in1   = Len_Trim (imp_problem_name)
 
      fname = imp_problem_name(1:in1) // suffix
 

      return

      end


c-----------------------------------------------------------------------------
c
c ROUTINE
c   Read_List
c
c DESCRIPTION
c   This routine reads items out of a file and puts them into an array.
c
c ARGUMENTS
c   list_items     : array to put the list items into
c   num_list_items : number of items in the list_items array
c   list_to_read   : file to read the list items from
c   max_list_items : maximum number of list items allowed
c
c-----------------------------------------------------------------------------

      subroutine Read_List
     &  (list_items, num_list_items, list_to_read, max_list_items)
 

c     ----------------------
c     Argument declarations.
c     ----------------------

      integer :: max_list_items
      integer :: num_list_items
      character (len=*) :: list_items(max_list_items)
      character (len=*) :: list_to_read
 

c     ----------------------
c     Variable declarations.
c     ----------------------

      character (len=75) :: err_msg
      character (len=80) :: tmp_string

      integer :: il
      integer :: iost
      integer :: rllun


c     ----------------
c     Begin execution.
c     ----------------

      err_msg(1:2)   = '  '

      num_list_items = 0


      call Asc_Open_Rd (rllun, list_to_read)


      READLOOP: do il = 1, max_list_items

        Read (rllun, *, iostat = iost) list_items(il)

c                      =============
        if (iost /= 0) exit READLOOP
c                      =============

        num_list_items = il

      end do READLOOP
 

      if (iost > 0) then

        err_msg = 'In Read_List:  Error reading file names #1.'

      else if (num_list_items == 0) then

        err_msg = 'In Read_List:  No file names.'

      else if (num_list_items == max_list_items) then

        Read (rllun, *, iostat = iost) tmp_string

        if (iost > 0) then

          err_msg = 'In Read_List:  Error reading file names #2.'

        else if (iost == 0) then

          err_msg = 'In Read_List:  Too many file names.'

        end if

      end if

 
      if (err_msg(1:2) == 'In') then

        call Do_Err_Out (err_msg, .true., 2, num_list_items,
     &                   max_list_items, 0, 0.0d0, 0.0d0)

      end if


      Close (rllun)
 
 
      return
 
      end


c-----------------------------------------------------------------------------
c
c ROUTINE
c   Check_Range_1d
c
c DESCRIPTION
c   This routine does a gross range check on the data values in a 1D array,
c   and exits if it finds one out of range.
c
c ARGUMENTS
c   var_name : the string name of var_1d
c   loc_proc : local processor #
c   d11, d12 : dimensions for var_1d
c   var_1d   : the specified 1D array
c   loval    : the lowest  allowed value
c   hival    : the highest allowed value
c
c-----------------------------------------------------------------------------
 
      subroutine Check_Range_1d
     &  (var_name, loc_proc, d11, d12, var_1d, loval, hival)
 
      implicit none
 

c     ----------------------
c     Argument declarations.
c     ----------------------
 
      character (len=*) :: var_name
      integer :: loc_proc
      integer :: d11, d12
      real*8  :: var_1d(d11:d12)
      real*8  :: loval, hival
 
 
c     ----------------------
c     Variable declarations.
c     ----------------------
 
      character (len=75) :: err_msg

      integer :: max1(1), min1(1)
 
 
c     ----------------
c     Begin execution.
c     ----------------
 
      min1(:) = Minloc (var_1d(d11:d12))
      max1(:) = Maxloc (var_1d(d11:d12))
 

      min1(1) = min1(1) + d11 - 1
      max1(1) = max1(1) + d11 - 1

 
      if ((var_1d(min1(1)) < loval) .or.
     &    (var_1d(max1(1)) > hival))
     &  then
 
        Write (6,*) min1(:), loc_proc
        Write (6,*) max1(:), loc_proc
 
        err_msg = 'Value out of range in Check_Range_1d, ' // var_name
 
        call Do_Err_Out (err_msg, .true., 0, 0, 0, 2, var_1d(min1(1)),
     &                   var_1d(max1(1)))
 
      end if
 

      return

      end


c-----------------------------------------------------------------------------
c
c ROUTINE
c   Check_Range_2d
c
c DESCRIPTION
c   This routine does a gross range check on the data values in a 2D array,
c   and exits if it finds one out of range.
c
c ARGUMENTS
c   var_name : the string name of var_2d
c   loc_proc : local processor #
c   d11, d12, d21, d22 : dimensions for var_2d
c   var_2d   : the specified 2D array
c   loval    : the lowest  allowed value
c   hival    : the highest allowed value
c
c-----------------------------------------------------------------------------
 
      subroutine Check_Range_2d
     &  (var_name, loc_proc, d11, d12, d21, d22, var_2d, loval, hival)
 
      implicit none
 

c     ----------------------
c     Argument declarations.
c     ----------------------
 
      character (len=*) :: var_name
      integer :: loc_proc
      integer :: d11, d12, d21, d22
      real*8  :: var_2d(d11:d12, d21:d22)
      real*8  :: loval, hival
 
 
c     ----------------------
c     Variable declarations.
c     ----------------------
 
      character (len=75) :: err_msg

      integer :: max2(2), min2(2)
 
 
c     ----------------
c     Begin execution.
c     ----------------
 
      min2(:) = Minloc (var_2d(d11:d12,d21:d22))
      max2(:) = Maxloc (var_2d(d11:d12,d21:d22))
 

      min2(1) = min2(1) + d11 - 1
      min2(2) = min2(2) + d21 - 1

      max2(1) = max2(1) + d11 - 1
      max2(2) = max2(2) + d21 - 1

 
      if ((var_2d(min2(1),min2(2)) < loval) .or.
     &    (var_2d(max2(1),max2(2)) > hival))
     &  then
 
        Write (6,*) min2(:), loc_proc
        Write (6,*) max2(:), loc_proc
 
        err_msg = 'Value out of range in Check_Range_2d, ' // var_name
 
        call Do_Err_Out
     &    (err_msg, .true., 0, 0, 0, 2,
     &     var_2d(min2(1),min2(2)), var_2d(max2(1),max2(2)))
 
      end if
 

      return

      end


c-----------------------------------------------------------------------------
c
c ROUTINE
c   Check_Range_3d
c
c DESCRIPTION
c   This routine does a gross range check on the data values in a 3D array,
c   and exits if it finds one out of range.
c
c ARGUMENTS
c   var_name : the string name of var_3d
c   loc_proc : local processor #
c   d11, d12, d21, d22, d31, d32 : dimensions for var_3d
c   var_3d   : the specified 3D array
c   loval    : the lowest  allowed value
c   hival    : the highest allowed value
c
c-----------------------------------------------------------------------------
 
      subroutine Check_Range_3d
     &  (var_name, loc_proc, d11, d12, d21, d22, d31, d32, var_3d,
     &   loval, hival)
 
      implicit none
 

c     ----------------------
c     Argument declarations.
c     ----------------------
 
      character (len=*) :: var_name
      integer :: loc_proc
      integer :: d11, d12, d21, d22, d31, d32
      real*8  :: var_3d(d11:d12, d21:d22, d31:d32)
      real*8  :: loval, hival
 
 
c     ----------------------
c     Variable declarations.
c     ----------------------
 
      character (len=75) :: err_msg

      integer :: max3(3), min3(3)
 
 
c     ----------------
c     Begin execution.
c     ----------------
 
      min3(:) = Minloc (var_3d(d11:d12,d21:d22,d31:d32))
      max3(:) = Maxloc (var_3d(d11:d12,d21:d22,d31:d32))
 

      min3(1) = min3(1) + d11 - 1
      min3(2) = min3(2) + d21 - 1
      min3(3) = min3(3) + d31 - 1

      max3(1) = max3(1) + d11 - 1
      max3(2) = max3(2) + d21 - 1
      max3(3) = max3(3) + d31 - 1

 
      if ((var_3d(min3(1),min3(2),min3(3)) < loval) .or.
     &    (var_3d(max3(1),max3(2),max3(3)) > hival))
     &  then
 
        Write (6,*) min3(:), loc_proc
        Write (6,*) max3(:), loc_proc
 
        err_msg = 'Value out of range in Check_Range_3d, ' // var_name
 
        call Do_Err_Out (err_msg, .true., 0, 0, 0, 2,
     &                   var_3d(min3(1),min3(2),min3(3)),
     &                   var_3d(max3(1),max3(2),max3(3)))
 
      end if
 

      return

      end


c-----------------------------------------------------------------------------
c
c ROUTINE
c   Check_Range_4d
c
c DESCRIPTION
c   This routine does a gross range check on the data values in a 4D array,
c   and exits if it finds one out of range.
c
c ARGUMENTS
c   var_name : the string name of var_4d
c   loc_proc : local processor #
c   d11, d12, d21, d22, d31, d32, d41, d42 : dimensions for var_4d
c   var_4d   : the specified 4D array
c   loval    : the lowest  allowed value
c   hival    : the highest allowed value
c
c-----------------------------------------------------------------------------
 
      subroutine Check_Range_4d
     &  (var_name, loc_proc, d11, d12, d21, d22, d31, d32, d41, d42,
     &   var_4d, loval, hival)
 
      implicit none
 

c     ----------------------
c     Argument declarations.
c     ----------------------
 
      character (len=*) :: var_name
      integer :: loc_proc
      integer :: d11, d12, d21, d22, d31, d32, d41, d42
      real*8  :: var_4d(d11:d12, d21:d22, d31:d32, d41:d42)
      real*8  :: loval, hival
 
 
c     ----------------------
c     Variable declarations.
c     ----------------------
 
      character (len=75) :: err_msg

      integer :: max4(4), min4(4)
 
 
c     ----------------
c     Begin execution.
c     ----------------
 
      min4(:) = Minloc (var_4d(d11:d12,d21:d22,d31:d32,d41:d42))
      max4(:) = Maxloc (var_4d(d11:d12,d21:d22,d31:d32,d41:d42))
 

      min4(1) = min4(1) + d11 - 1
      min4(2) = min4(2) + d21 - 1
      min4(3) = min4(3) + d31 - 1
      min4(4) = min4(4) + d41 - 1

      max4(1) = max4(1) + d11 - 1
      max4(2) = max4(2) + d21 - 1
      max4(3) = max4(3) + d31 - 1
      max4(4) = max4(4) + d41 - 1

 
      if ((var_4d(min4(1),min4(2),min4(3),min4(4)) < loval) .or.
     &    (var_4d(max4(1),max4(2),max4(3),max4(4)) > hival))
     &  then
 
        Write (6,*) min4(:), loc_proc
        Write (6,*) max4(:), loc_proc
 
        err_msg = 'Value out of range in Check_Range_4d, ' // var_name
 
        call Do_Err_Out (err_msg, .true., 0, 0, 0, 2,
     &                   var_4d(min4(1),min4(2),min4(3),min4(4)),
     &                   var_4d(max4(1),max4(2),max4(3),max4(4)))
 
      end if
 

      return

      end


c-----------------------------------------------------------------------------
c
c ROUTINE
c   Check_Range_5d
c
c DESCRIPTION
c   This routine does a gross range check on the data values in a 5D array,
c   and exits if it finds one out of range.
c
c ARGUMENTS
c   var_name : the string name of var_5d
c   loc_proc : local processor #
c   d11, d12, d21, d22, d31, d32, d41, d42, d51, d52 : dimensions for var_5d
c   var_5d   : the specified 5D array
c   loval    : the lowest  allowed value
c   hival    : the highest allowed value
c
c-----------------------------------------------------------------------------
 
      subroutine Check_Range_5d
     &  (var_name, loc_proc, d11, d12, d21, d22, d31, d32, d41, d42,
     &   d51, d52, var_5d, loval, hival)
 
      implicit none
 

c     ----------------------
c     Argument declarations.
c     ----------------------
 
      character (len=*) :: var_name
      integer :: loc_proc
      integer :: d11, d12, d21, d22, d31, d32, d41, d42, d51, d52
      real*8  :: var_5d(d11:d12, d21:d22, d31:d32, d41:d42, d51:d52)
      real*8  :: loval, hival
 
 
c     ----------------------
c     Variable declarations.
c     ----------------------
 
      character (len=75) :: err_msg

      integer :: max5(5), min5(5)
 
 
c     ----------------
c     Begin execution.
c     ----------------
 
      min5(:) = Minloc (var_5d(d11:d12,d21:d22,d31:d32,d41:d42,d51:d52))
      max5(:) = Maxloc (var_5d(d11:d12,d21:d22,d31:d32,d41:d42,d51:d52))
 

      min5(1) = min5(1) + d11 - 1
      min5(2) = min5(2) + d21 - 1
      min5(3) = min5(3) + d31 - 1
      min5(4) = min5(4) + d41 - 1
      min5(5) = min5(5) + d51 - 1

      max5(1) = max5(1) + d11 - 1
      max5(2) = max5(2) + d21 - 1
      max5(3) = max5(3) + d31 - 1
      max5(4) = max5(4) + d41 - 1
      max5(5) = max5(5) + d51 - 1

 
      if ((var_5d(min5(1),min5(2),min5(3),min5(4),min5(5)) < loval) .or.
     &    (var_5d(max5(1),max5(2),max5(3),max5(4),max5(5)) > hival))
     &  then
 
        Write (6,*) min5(:), loc_proc
        Write (6,*) max5(:), loc_proc
 
        err_msg = 'Value out of range in Check_Range_5d, ' // var_name
 
        call Do_Err_Out
     &    (err_msg, .true., 0, 0, 0, 2,
     &     var_5d(min5(1),min5(2),min5(3),min5(4),min5(5)),
     &     var_5d(max5(1),max5(2),max5(3),max5(4),max5(5)))
 
      end if
 

      return

      end


c-----------------------------------------------------------------------------
c
c ROUTINE
c   Wrap_Master_2d
c
c DESCRIPTION
c   This routine wraps the 2D ghost zone values for the master processor only.
c   It operates on an entire array.
c
c ARGUMENTS
c   var_2d : the specified 2D array
c
c-----------------------------------------------------------------------------
 
      subroutine Wrap_Master_2d (var_2d)

      use h_imp_dims 
      implicit none
 
c#     include "imp_dims.h"
 

c     ----------------------
c     Argument declarations.
c     ----------------------
 
      real*8  :: var_2d(ilo:ihi, julo:jhi)
 
 
c     ----------------------
c     Variable declarations.
c     ----------------------
 
      integer :: il
 
 
c     ----------------
c     Begin execution.
c     ----------------
 
      do il = 1, imp_nborder
 
        var_2d(i1-il,ju1:j2) = var_2d(i2-il+1,ju1:j2)

        var_2d(i2+il,ju1:j2) = var_2d(i1+il-1,ju1:j2)
 
      end do
 
 
      return
 
      end


c-----------------------------------------------------------------------------
c
c ROUTINE
c   Wrap_Master_3du
c
c DESCRIPTION
c   This routine wraps the 3D/julo ghost zone values for the master processor
c   only.  It operates on an entire array.
c
c ARGUMENTS
c   var_3du : the specified 3D/julo array
c
c-----------------------------------------------------------------------------
 
      subroutine Wrap_Master_3du (var_3du)
      use h_imp_dims
      implicit none
 
c#     include "imp_dims.h"
 

c     ----------------------
c     Argument declarations.
c     ----------------------
 
      real*8  :: var_3du(ilo:ihi, julo:jhi, k1:k2)
 
 
c     ----------------------
c     Variable declarations.
c     ----------------------
 
      integer :: il
 
 
c     ----------------
c     Begin execution.
c     ----------------
 
      do il = 1, imp_nborder
 
        var_3du(i1-il,ju1:j2,:) = var_3du(i2-il+1,ju1:j2,:)

        var_3du(i2+il,ju1:j2,:) = var_3du(i1+il-1,ju1:j2,:)
 
      end do
 
 
      return
 
      end


c-----------------------------------------------------------------------------
c
c ROUTINE
c   Wrap_Master_3dv
c
c DESCRIPTION
c   This routine wraps the 3D/jvlo ghost zone values for the master processor
c   only.  It operates on an entire array.
c
c ARGUMENTS
c   var_3dv : the specified 3D/jvlo array
c
c-----------------------------------------------------------------------------
 
      subroutine Wrap_Master_3dv (var_3dv)
      use h_imp_dims 
      implicit none
 
c#     include "imp_dims.h"
 

c     ----------------------
c     Argument declarations.
c     ----------------------
 
      real*8  :: var_3dv(ilo:ihi, jvlo:jhi, k1:k2)
 
 
c     ----------------------
c     Variable declarations.
c     ----------------------
 
      integer :: il
 
 
c     ----------------
c     Begin execution.
c     ----------------
 
      do il = 1, imp_nborder
 
        var_3dv(i1-il,jv1:j2,:) = var_3dv(i2-il+1,jv1:j2,:)

        var_3dv(i2+il,jv1:j2,:) = var_3dv(i1+il-1,jv1:j2,:)
 
      end do
 
 
      return
 
      end


c-----------------------------------------------------------------------------
c
c ROUTINE
c   Wrap_Master_4d
c
c DESCRIPTION
c   This routine wraps the 4D ghost zone values for the master processor only.
c   It operates on an entire array.
c
c ARGUMENTS
c   var_4d : the specified 4D array
c   num_d4 : number of elements in 4th dimension
c
c-----------------------------------------------------------------------------
 
      subroutine Wrap_Master_4d (var_4d, num_d4)

      use h_imp_dims 
      implicit none
 
c#     include "imp_dims.h"
 

c     ----------------------
c     Argument declarations.
c     ----------------------
 
      integer :: num_d4
      real*8  :: var_4d(ilo:ihi, julo:jhi, k1:k2, num_d4)
 
 
c     ----------------------
c     Variable declarations.
c     ----------------------
 
      integer :: il
 
 
c     ----------------
c     Begin execution.
c     ----------------
 
      do il = 1, imp_nborder
 
        var_4d(i1-il,ju1:j2,:,:) = var_4d(i2-il+1,ju1:j2,:,:)

        var_4d(i2+il,ju1:j2,:,:) = var_4d(i1+il-1,ju1:j2,:,:)
 
      end do
 
 
      return
 
      end

