 
c=============================================================================
c  111809      Guangxing Lin, Sandy Sillman
c       To take cloud fraction into acccount when calculating the aqueous reaction
c  in clouds.
c  
cDEBUG WRITE and WRITE OUTPUT: 
c TEST CHEM:  OUTPUT FOR SPECIFIC GRID. (coordinate w/ quadchem_solver)
c   also in quadchem_solver,  chem_update and  Tuv_lookup.F 
c
c DEBUG 807 WRITES added by Sandy Sillman - CUT
c   (see VERSION_DEBUG92107 and 92207  for  DEBUG statements)
c
c $Id: qk_update.F,v 1.3 2003/05/16 22:22:51 akinorii Exp $
c
c CODE DEVELOPER
c   John Tannahill, LLNL
c   jrt@llnl.gov
c
c   IMPACT quadchem modifications:  Akinori Ito.
c
c FILE
c   qk_update.F
c
c ROUTINES
c   Update_Qk
c   Calc_Lwc
c   Calc_Qk_Stats
c
c=============================================================================
 
 
c-----------------------------------------------------------------------------
c
c ROUTINE
c   Update_Qk
c
c DESCRIPTION
c   This routine updates the thermal rate constants (i.e., qk's).
c
c ARGUMENTS
c   do_wetchem         : do wet chemistry?
c   metdata_name_org   : first  part of metdata_name, e.g., "NCAR"
c   metdata_name_model : second part of metdata_name, e.g., "MATCH"
c   num_time_steps     : number of time steps
c   ih2o_num           : const array index for water vapor
c   imgas_num          : const array index for air density
c   nymd               : current year/month/day (YYMMDD)
c   rxnr_adjust_map    : mapping of reaction rate adjustment number to
c                        reaction rate number
c   latdeg      : latitude of grid centers (deg)
c   pres3c      : atmospheric pressure at the center of each grid box (mb)
c   tropp       : tropopause pressure (mb)
c   temp3       : temperature (degK)
c   clwc        : cloud liquid water content, grid box average (gm/m^3)
c   cmf1        : convective mass flux used for cloud fraction calculation
c                 (kg/m2/s)
c   sadimp      : surface area densities (cm^2/cm^3)
c   qkimp       : thermal rate constants (units vary)
c
c Added by Sandy Sillman 2007
c  OPTION:  declare here and in chem_update or in imp_arrays_big
c   qhimp       : Henry's law constants (mol lit-1 atm-1)
c   qhimp_temp  : Henry's law constants (units?)
c   qaqimp       : Aqueous equilibrium constants (mol lit-1)
c   qaqqimp       : Aqueous special equilibrium constants (mol lit-1)
c
c   const       : species concentration, known at zone centers
c                 (mixing ratio)
c   rxnr_adjust : array of reaction rate adjustment factors
c   chem_opt    : chemistry option
c 
c Added by Sandy Sillman, 2007:  water vapor and cloud liquid water. watv
c   watv_imp    :  water vapor, molec/cm3 (from Qk_update)
c   clwc_imp    :  cloud liquid water, g/cm3 (from Qk_update)
c
c-----------------------------------------------------------------------------

      subroutine Update_Qk
     &  (do_wetchem, metdata_name_org, metdata_name_model,
     &   num_time_steps, ih2o_num, imgas_num, nymd, 
     &   latdeg, pres3c,  temp3, cmf1, sadimp, qkimp,
     &    qhimp,qhimp_temp, qaqimp, qaqqimp,
     &      watv_imp, clwc_imp,clfrac_imp,
c-----------Guangxing Lin,111609,to modify  Calc_Lwc
     &   moistq, 
c
     &   const,  chem_opt)
c
c- delete rxnr_adjust and rxnr_adjust_map, clwc,since they are not used in
c the code
      use h_imp_time_constants
      use h_imp_dims
      use h_rbchemmech
      use h_sulfquad
      use h_setkin_par
      use h_setkin_mw 
      use abortutils,    only: endrun

      use h_imp_phys_constants, only: MWTAIR, AVOGAD
      implicit none

c#     include "imp_time_constants.h"
c#     include "imp_dims.h"

c
c Added by Sandy Sillman, 2007 (to pass chem. mechanism to rate calc.)
c#     include "rbchemmech.h"
c ********************************************
c ** Begin of modifications by Luis Olcese, UM
c#     include "sulfquad.h"
c#     include "setkin_par.h"
c#     include "setkin_mw.h"
c ** End of modifications by Luis Olcese, UM
c ********************************************

c     ----------------------
c     Argument declarations.
c     ----------------------

      logical :: do_wetchem
      character*16 :: metdata_name_org
      character*16 :: metdata_name_model
      integer :: num_time_steps
      integer :: ih2o_num
      integer :: imgas_num
      integer :: nymd
c      integer :: rxnr_adjust_map(*)

      integer :: chem_opt

      real*8  :: latdeg  (ju1_gl:j2_gl)
      real*8  :: pres3c  (ilo:ihi, julo:jhi, k1:k2)
c      real*8  :: tropp   (i1:i2,   ju1:j2)
      real*8  :: temp3   (ilo:ihi, julo:jhi, k1:k2)
      real*8  :: cmf1    (i1:i2,   ju1:j2,   k1:k2)
      real*8  :: sadimp  (i1:i2,   ju1:j2,   k1:k2, num_sad)
      real*8  :: qkimp   (i1:i2,   ju1:j2,   k1:k2, num_qks)
      real*8  :: qhimp   (i1:i2,   ju1:j2,   k1:k2, num_qhs)
      real*8  :: qhimp_temp (i1:i2,   ju1:j2,   k1:k2, num_qhs)
      real*8  :: qaqimp  (i1:i2,  ju1:j2,   k1:k2, num_qaqs)
      real*8  :: qaqqimp (i1:i2, ju1:j2,   k1:k2, num_qaqqs)
      real*8  :: const   (i1:i2,   ju1:j2,   k1:k2, num_species)
c      real*8  :: rxnr_adjust(i1:i2, ju1:j2,k1:k2,
c     &                       num_rxnr_adjust, rxnr_adjust_timpyr)
c      real*8  :: clwc    (i1:i2,   ju1:j2,   k1:k2)

      real*8  :: watv_imp     (i1:i2, ju1:j2, k1:k2)
      real*8  :: clwc_imp     (i1:i2, ju1:j2, k1:k2)
      real*8  :: clfrac_imp     (i1:i2, ju1:j2, k1:k2)

      real*8  :: moistq  (i1:i2,  ju1:j2, k1:k2)
!      real*8  :: clwc2  (i1:i2,  ju1:j2, k1:k2)
!      real*8  :: ran_cloud  (i1:i2,  ju1:j2, k1:k2)
c     -----------------------
c     Parameter declarations.
c     -----------------------

      logical, parameter :: DO_QK_STATS = .false.


c     ----------------------
c     Variable declarations.
c     ----------------------

      logical, save :: first = .true.

      integer :: idumyear, idumday
      integer :: il, ij, ik, ic
      integer :: im, iq

      real*8  :: adcol   (k1:k2)
      real*8  :: prescol (k1:k2)
      real*8  :: tempcol (k1:k2)

      real*8  :: cmfcol  (k1:k2)
      real*8  :: lwccol  (k1:k2)
      real*8  :: watcol  (k1:k2)

      real*8  :: moistqcol (k1:k2)
      real*8  :: clfrac (k1:k2)

      real*8  :: constcol(num_molefrac, k1:k2)
      real*8  :: qkcol   (num_qks,      k1:k2)
      real*8  :: sadcol  (num_sad,      k1:k2)

      real*8  :: temp(1), dens(1), watv(1), watliq(1)
c      real*8     watliq_t, saq_aer_t 
      real*8  :: sad_lbs(1)
      real*8  :: clfrac_temp
cc-------------------gxlin------07172009-----------------
      real*8  :: saq_aer(1) !added to include heterogenous reactions on aqueous aerosol surface
      real*8  :: pii 
c---------------------gxlin---------------------------

c----------gxlin-----07-29-2009-----------
c      real*8  ::  rh   !relative humidity
c------gxlin ----07-29-2009----------------

c ********************************************
c ** Begin of modifications by Luis Olcese, UM
c ** Changed to include secondary organic aerosols
      real*8 :: poa_IFOC, poa_IBOC, poa_mw_IFOC, poa_mw_IBOC, poa, soa,
     &          numb_mol
      real*8 :: s_mwaver, s_m0

      real*8 :: ugm3
c---------------gxlin---------------------------
c--     added by Sandy and gxlin -07022009
      real*8 :: density, r_cld,rho_water

c------------gxlin------------------------------
c ** End of modifications by Luis Olcese, UM
c ********************************************

      real*8, allocatable :: ratek(:,:) 
      real*8, allocatable :: rateh(:,:) 
      real*8, allocatable :: rateh_temp(:,:) 
      real*8, allocatable :: rateq(:,:) 
      real*8, allocatable :: rateqq(:,:) 

       integer :: ierror(100),ierr_count
       
      ierr_count =0
      ierr_count = ierr_count+1
      Allocate (ratek(1:1, 1:num_qks),stat=ierror(ierr_count))
      ierr_count = ierr_count+1
      Allocate (rateh(1:1, 1:num_qhs),stat=ierror(ierr_count))
      ierr_count = ierr_count+1
      Allocate (rateh_temp(1:1, 1:num_qhs),stat=ierror(ierr_count))
      ierr_count = ierr_count+1
      Allocate (rateq(1:1, 1:num_qaqs),stat=ierror(ierr_count))
      ierr_count = ierr_count+1
      Allocate (rateqq(1:1, 1:num_qaqqs),stat=ierror(ierr_count))

        if ( any( ierror(1:ierr_count) .ne. 0 ) ) then
           print*, ierror(1:ierr_count)
           call endrun("Error in allocation in qk_Update")

         end if

      ratek = 0.0d0
      rateh = 0.0d0
      rateh_temp = 0.0d0
      rateq = 0.0d0
      rateqq = 0.0d0
      pii = 3.141592654d0 
c     ----------------
c     Begin execution.
c     ----------------
 
      if (pr_diag) then
        Write (6,*) 'Update_Qk called by ', loc_proc
        write(6,*) 'i1,i2,ju1,j2,k1,k2 =', 
     &           i1,i2,ju1,j2,k1,k2
      end if

c ********************************************
c ** Begin of modifications by Luis Olcese, UM
c ** From chemistry version
c     =======================
c      if (chem_opt == 7) then
      if (chem_opt == 7 .or. chem_opt == 9) then
c     =======================
c ** End of modifications by Luis Olcese, UM
c ********************************************

      do ij = ju1, j2
         do il = i1, i2

c
c Added by Sandy Sillman, 2007: 
c  calculate LWC for use in setting rates, chem:  copied from below

          prescol(:) = pres3c(il,ij,:)
          tempcol(:) = temp3 (il,ij,:)

          moistqcol(:) = moistq(il,ij,:) 
c--------Guangxing Lin---11072012-------------
c use the in-cloud water content and cloud fraction from CAM5 
c so comment out these 2 lines below

!          clwc_imp(il,ij,:) = 0.d0 
!          clfrac_imp(il,ij,:) = 0.d0 
c--------Guangxing Lin---11072012-------------
c-------------gxlin-----------07182009-------------
c          if (do_wetchem) then
c            if ((metdata_name_org  (1:4) == 'NCAR') .and.
c     &          (metdata_name_model(1:4) == 'CCM3')) then
c              lwccol(:) = clwc(il,ij,:)
c            else
              cmfcol(:) = cmf1(il,ij,:) * 3600.0d0 * 9.8d0 /100.0d0    
              watcol(:) = const(il,ij,:,ih2o_num)
c-------------gxlin----------
c        if (pr_diag) then
c         Write (6,*) 'test test Update_Qk called by ', loc_proc
c        end if
c-----gxlin----
c--------Guangxing Lin---11072012-------------
!------------use CAM5 lwc, no need below any more
!c             =============
!              call Calc_Lwc
!c             =============
!     &          (ij, latdeg, cmfcol, prescol, tempcol, watcol, lwccol,
!     &             moistqcol, clfrac)

!            end if
c     --------------------------------------------------------------



cccccc----------------------gxlin-----------------------
c Added by Sandy Sillman, 2007:  
c    Store clwc_imp for use in quad chemistry.
c   Converted from gram/m3 to gram/cm3
c--------Guangxing Lin---11072012-------------
c use the in-cloud water content and cloud fraction from GFDL
c so comment out these 2 lines below

!            clwc_imp(il,ij,:) = lwccol(:) * 1.0E-06
!            clfrac_imp(il,ij,:) = clfrac(:) 

c--------Guangxing Lin---11072012-------------

c          end if

            do ik = k1, k2

c Set parameters for use in reaction rate calculation

            temp(1) = temp3 (il,ij,ik)
            dens(1) = const(il,ij,ik,imgas_num)
            watv(1) = dens(1) * const(il,ij,ik, ih2o_num)
            watliq(1) = clwc_imp(il, ij, ik) 
            !watliq(1) = 0.0d0 !Guangxing Lin debug 

            sad_lbs(1) = sadimp(il,ij,ik,1)

c--------------gxlin----------------07-29-2009-----------
c     --------------------------------------------------------------
c     Calculate relative humidity from Seinfeld (1986) p. 181.
c     The first  rh is the temperature dependent parameter a.
c     The second rh is the saturation vapor pressure of water.
c     The third  rh is the actual relative humidity as a fraction.
c     Then make sure rh is between 0 and 1.
c     --------------------------------------------------------------            
c            rh = 1.0d0 - (373.15d0 / temp(1))
c
c            rh =
c     &      1013.25d0 * Exp (13.3185d0 * rh -
c     &                    1.9760d0 * rh**2 -
c     &                    0.6445d0 * rh**3 -
c     &                    0.1299d0 * rh**4)

c            rh = watcol(ik) * prescol(ik) / rh

c            rh = Max (Min (rh, 1.0d0), 0.0d0)
c----------------------------gxlin----07-29-2009-------- 
c Added by Sandy Sillman ,2007:  Save water vapor for use later
            watv_imp(il,ij,ik) = watv (1)

c ********************************************
c ** Begin of modifications by Luis Olcese, UM
c ** Changed to include secondary organic aerosols

            poa_IFOC = const(il,ij,ik,IFOC) + 
     &            const(il,ij,ik,IOCPHl1)+ const(il,ij,ik,IOCPHl2)
c======Guangxing Lin== add two more OC: from 3 BC/OC scheme
            poa_IBOC = const(il,ij,ik,IBOC)

            poa_mw_IFOC = mw_data(IFOC)
            poa_mw_IBOC = mw_data(IBOC)
c--------------------------gxlin------------------------
c------------- get air density in molecules/cubic cm ---07022009

            density = dens(1)
c------------------------------------gxlin-------------
 
           poa = max(ugm3(poa_IFOC,poa_mw_IFOC,density) +
     &       ugm3(poa_IBOC,poa_mw_IBOC,density), 1.d-3)

            soa = 0.d0
            numb_mol = 0.d0
            do ic = soa_ind_ini, soa_ind_end
              soa = soa + ugm3(const(il,ij,ik,ic),mw_data(ic),density)
              numb_mol = numb_mol + const(il,ij,ik,ic)*density
            enddo

            s_m0 = poa + soa

            if (numb_mol < 1.d5) then
              s_mwaver = 150.d0
            else
              s_mwaver = 0.d0
              do ic = soa_ind_ini, soa_ind_end
               s_mwaver=s_mwaver+
     $          (const(il,ij,ik,ic)*density*mw_data(ic))/numb_mol
              enddo
            endif


c ** End of modifications by Luis Olcese, UM
c ********************************************
c---------------------gxlin ---07172009---------------------
c
c Set parameters for use in reaction rate calculation (continued):
c
c Added by Sandy Sillman,63009 2009:
c   aqueous aerosol surface area (saq_aer)
c   Temporarily set equal to sulfate aerosol (sad_lbs).
c
c  Modify as function of sulfate aerosol (sad_lbs),
c     water vapor/rel humidity (watv)
c       (watv set from 'humidity' in Add_Trop_Water)
c         see also Calc_Humidity_Rel and relhum in imp_step.F and imp_step_subs)
c     cloud liquid water (clwc) and droplet radius,
c     prior soa (or soluble soa)

c ORIGINAL VERSION: aqueous surface area = sulfate surface area
c            saq_aer(1) = sad_lbs(1)

c CLOUD WATER SURFACE AREA = liq. water + diameter + unit conversion
c
c   saq_aer surface area  in cm2/cm3
c
c   cloud droplet radius = 1.000E-03  cm
c
c   clwc in gram/cm3  = cm3/cm3
c
c   surface area = pi r^2  volume = 4/3 pi r^3
c
c   (frozen droplets? no - lwc=liq water concentration)

               r_cld=1.0d-3                !cm
               rho_water=1.0d0             !g/cm3
             saq_aer(1)= watliq(1)/rho_water*4.0d0*pii*r_cld**2.d0
     $         /(4.0d0/3.0d0*pii*r_cld**3.0d0)

!             clfrac_temp = clfrac(ik)
             clfrac_temp = clwc_imp(il,ij,ik) !Guangxing Lin 
!             clfrac_temp=0.0d0 !debug 
 
c             write(6,*) "cloud surface area: ", saq_aer(1) 
c-------------gxlin---11-07-2009--------------
c       seperate ne_MGLY formed from reaction on sulfate aerosol vs from clouds
c             saq_aer(1) = saq_aer(1)+sad_lbs(1)
c             write(6,*) "sulfate surface aera: ", sad_lbs(1)
c------------------------------------------------------------
c -----------gxlin -------07-29-2009---------------------
c ---- calculate the aqueous aerosol surface area
c ---- using relative humidity-dependent effective radius 
c      1, sulfate
c            Nt = const(il,ij,ik,7)      
c            call CalcR_eff(rh,N)
c            saq_aer(1)=N*pii*r_eff**2.0
c------------gxlin-----------07-29-2009----------------
c Relative humidity
c WATER VAPOR  SATURATION H2O  BASED ON GILL, p. 605-6.
c  saturation vapor pressure in milibars Ew
c        xh2osat = 10.**( (0.7859 + 0.03477*(temp(1)-273.) )
c    *                 /(1.    + 0.00412*(temp(1)-273.) )    )
c        xh2osat = xh2osat *(1.  + 0.001 * (dens(1)/2.46E19)
c    *                       * (4.5 +0.0006*(temp(1)-273.)**2)  )
c (convert from mb to molec/cm3 at surface pressure:)
c        xh2osat = xh2osat * 2.44e+16 *dens(1)/2.46E19

c (1000 mb = 2.46E19 molec/cm3? check.)
c
c   watv =   water vapor, molec/cm3 (from Qk_update)
c
c RH = watv/xh2osat
c--------------------------gxlin---------------------------------

c           ===========
c           call Brates
c    &     (temp, dens, watv, ratek,rateh, rateq, rateqq, sad_lbs)
c           ===========
            call   rbchemrates
c           ===========
     &   (temp, dens, watv,watliq, sad_lbs,      
c--------------gxlin---------07172009------------------
c    to include the heterogenous reaction on aqueous aerosol surface

     &                    clfrac_temp, saq_aer

c----------------gxlin-----------------------------------
c ********************************************
c ** Begin of modifications by Luis Olcese, UM
c ** Changed to include secondary organic aerosols
c     &   ,ratek, rateh, rateq, rateqq)
     &   ,ratek, rateh,rateh_temp, rateq, rateqq, s_m0, s_mwaver)
c ** End of modifications by Luis Olcese, UM
c ********************************************



c=====Guangxing Lin===09/25/2014====debug test only======
c       if(pres3c(il,ij,ik).le.400.0) then
c          ratek(1,537)=0.0
c          ratek(1,540)=0.0
c          ratek(1,543)=0.0
c          ratek(1,546)=0.0
c          ratek(1,552)=0.0
c          ratek(1,555)=0.0
c          ratek(1,558)=0.0
c          ratek(1,561)=0.0
c          ratek(1,564)=0.0
c          ratek(1,567)=0.0
c          ratek(1,570)=0.0
c          ratek(1,573)=0.0
c          ratek(1,576)=0.0
c          ratek(1,579)=0.0
c          ratek(1,582)=0.0
c          ratek(1,585)=0.0
c          ratek(1,588)=0.0
c          ratek(1,591)=0.0
c          ratek(1,594)=0.0
c          ratek(1,597)=0.0
c          ratek(1,604)=0.0
c          ratek(1,607)=0.0
c          ratek(1,610)=0.0
c          ratek(1,613)=0.0
c          ratek(1,616)=0.0
c        end if
c=====Guangxing Lin===09/25/2014====debug test only======
          
c TEST CHEM WRITE:

c TEST CHEM FOR GLYX ne_GLYX, 10-05-2009, gxlin 

      if(
c ANTARCTICA
c    &         ( il.eq.66.and. ij.eq. 2.and.ik.eq.1            ) )
     &         ( il.eq.66.and. ij.eq. 2.and.ik.eq.2  ) 
c
c OHIO
c     &         (il.eq.56.and. ij.eq.33.and.(ik.eq.1 .or.ik.eq.4 ) )
c     &         (il.eq.41.and. ij.eq.37.and.(ik.eq.2 ) )
c GERMANY
c     &    .or. (il.eq. 2.and. ij.eq.36.and.(ik.eq.1            ) )
c SIBERIA/BAIKAL 55 N 100 E
c     &    .or. (il.eq.20.and. ij.eq.37.and.(ik.eq.1            ) )
c SOUTHERN BRAZIL/PARAGUAY/  25 S 60 W
c     &    .or. (il.eq.60.and. ij.eq.17.and.(ik.eq.1            ) )
c NORTH ATLANTIC
c     &    .or. (il.eq.60.and. ij.eq.33.and.(ik.eq.1            ) )

     &               ) then


          write(c_out, 301) il, ij, ik
301      format(/'TEST RATES IN qk_update: LON, LAT, SIG il,ij,ik= ',
     &                     3i3)

           write(c_out , 303)  watv(1), saq_aer(1)
     &                      ,watliq(1), sad_lbs(1),clfrac_temp 
303        format(  '   watv, saq_aer, CLWC, 
     $      sad_lbs,clfrac =',/,(5(1pe10.3)) )
 
         write(c_out, 307)              ratek(1,553)
         write(c_out, 308)              ratek(1,554)
307      format('GLYX   ->  ne_GLYX :     ratek= ', 4(1pe10.3))
308      format('MGLY   ->  ne_MGLy :     ratek= ', 4(1pe10.3))

       endif  ! TEST CHEM 6/23/09



c      coordinate this with TEST CHEM WRITE in quadchem_solver.F

          if(

c OPTION FOR NO TEST CHEM WRITE
     &         (il.eq.-305)
cic
c ANTARCTICA
c     &         ( il.eq.66.and. ij.eq. 2.and.(ik.eq.2     )) .or. 
c OPTION FOR SPECIFIC GRIDS
c   NOTE - Coordinate this with TEST CHEM output  in qk_update.F 
c OHIO
c     &         (il.eq.56.and. ij.eq.33.and.(ik.eq.1.or.ik.eq.4 ) )

c     &         (il.eq.36.and.ij.eq.2                           )
c     &   .or. (il.eq.59.and. ij.eq. 6                           )
c    &   .or. (il.eq.66.and. ij.eq. 9.and.(ik.eq.1.or.ik.eq.14) )
c     &           (il.eq.47.and.ij.eq.32.and.ik.eq.1)
     &                                           )then
           write(6    , 201) il, ij, ik
201        format(/'TEST CHEM in qk_update:        il,ij,ik= ',/,3i3)
c         write(c_out, 203) s_m0, s_mwaver
c203      format(/,'  s_m0, s_mvaver   =',(3(1pe10.3)) )

          write(c_out,*) 'poa, soa, s_m0 =', poa, soa, s_m0
          write(c_out,*) 'soa_ind_ini, soa_ind_end =',
     &        soa_ind_ini, soa_ind_end
           write(6    , 203) temp(1), dens(1), watv(1)
     &                      ,watliq(1), sad_lbs(1) 
203        format(  '  temp, dens, watv, CLWC, saer =',/,(5(1pe10.3)) )
           write(6    ,205) ratek(1, 1), ratek(1,   22), 
     &                      ratek(1,229), 
     &                      ratek(1, 553), ratek(1,554 )
205       format(' RATE 1, 22,229, 553, 554=',/,(5(1pe10.3)) )
         endif   ! END TEST CHEM WRITE

croy
c         write(42,*) 'c_nreac,c_nreacq,c_nreach,num_qks=',
c     &         c_nreac,c_nreacq,c_nreach,num_qks
croy

c           write(6,*) ' num_qks,', num_qks
c           write(6,*)' gxlin-test-ratek553,ratek554', ratek(1,553), ratek(1,554)
            do iq = 1, num_qks
              qkimp(il,ij,ik,iq) = ratek(1,iq)
            enddo
            do iq = 1, num_qhs
              qhimp(il,ij,ik,iq) = rateh(1,iq)
              qhimp_temp(il,ij,ik,iq) = rateh_temp(1,iq)
            enddo
            do iq = 1, num_qaqs
              qaqimp(il,ij,ik,iq) = rateq(1,iq)
            enddo
            do iq = 1, num_qaqqs
              qaqqimp(il,ij,ik,iq) = rateqq(1,iq)
            end do
           end do
        end do
      end do


c     =======================
      end if
c     =======================


      Deallocate (ratek)
      Deallocate (rateh)
      Deallocate (rateh_temp)
      Deallocate (rateq)
      Deallocate (rateqq)
      if (pr_diag) then
        Write (6,*) 'Finished Update_Qk called by ', loc_proc
      end if
      return

      end


c-----------------------------------------------------------------------------
c
c ROUTINE
c   Calc_Lwc

c DESCRIPTION
c   This routine calculates liquid water in each grid box in a single column.
c   It uses parameterizations of cloud water from Kiehl and cloud
c   fractions from Xu.
c
c ARGUMENTS
c   ij      : latitude index for this column
c   latdeg  : latitude of grid box center   (deg)
c   cmfcol  : convective mass flux used for cloud fraction calculation
c             (gm/cm^2/hour ???)
c   prescol : atmospheric pressure at the center of each grid box (mb)
c   tempcol : temperature (degK)
c   watcol  : water vapor concentration, known at zone centers (mixing ratio)
c   lwccol  : liquid water in each grid box (gm/m3)
c   11-16-2009,   Guangxing Lin, Sandy,
c   
c   Modify this subroutine based on the calculation of liquid water in sulfur
c   chemistry.   
c
c-----------------------------------------------------------------------------

      subroutine Calc_Lwc
     &  (ij, latdeg, cmfcol, prescol, tempcol, watcol, lwccol,
     &    moistqcol, clfrac)

      use h_imp_phys_constants
      use h_imp_dims
      implicit none

c#     include "imp_phys_constants.h"
c#     include "imp_dims.h"


c     ----------------------
c     Argument declarations.
c     ----------------------

      integer :: ij,ik
      real*8  :: latdeg (ju1_gl:j2_gl)
      real*8  :: cmfcol (k1:k2)
      real*8  :: prescol(k1:k2)
      real*8  :: tempcol(k1:k2)
      real*8  :: watcol (k1:k2)
      real*8  :: lwccol (k1:k2)

      real*8  :: moistqcol (k1:k2)
      real*8  :: clfrac(k1:k2)

c     ----------------------
c     Variable declarations.
c     ----------------------

      real*8  :: c0 (k1:k2)
      real*8  :: c1 (k1:k2)
      real*8  :: c2 (k1:k2)

      real*8  :: cfc(k1:k2)
      real*8  :: cfs(k1:k2)

      real*8  :: rh (k1:k2)
      real*8  :: rhc(k1:k2)

      real*8  :: height_asl(k1:k2)  ! height above sea level (m)


c     ----------------
c     Begin execution.
c     ----------------

      if (pr_diag) then
        Write (6,*) 'Calc_Lwc called by ', loc_proc
      end if


      c0 (:) = 0.0d0; c1(:) = 0.0d0; c2(:) = 0.0d0

      rhc(:) = 0.0d0


      height_asl(:) =
     &  -29.3d0 * tempcol(:) * Log (prescol(:) / 1013.25d0)

      height_asl(:) = Max (height_asl(:), 0.0d0)


c     -------------------------------------------------------------------
c     Calculate cloud water from a parameterization of Kiehl found in
c     "Sensitivity of the Simulated Climate to a Diagnostic Formulation
c     for Cloud Liquid Water" by James Hack, Journal of Climate, Vol. 11,
c     July 1998, p 1499.
c     -------------------------------------------------------------------

      lwccol(:) =
     &  0.18d0 *
     &  Exp (-height_asl(:) /
     &       (1080.0d0 + (2000.0d0 * Cos (latdeg(ij) * RADPDEG)**2)))


c     ----------------------------------------------------------------
c     Calculate convective and stratiform cloud fractions from a
c     parameterization of Xu found in "Evaluation of Cloudiness
c     Parameterizations Using a Cumulus Ensemble Model" by Kuan-Man Xu
c     and Steven Krueger, Monthly Weather Review, Vol. 119 p 342.
c     ----------------------------------------------------------------

c     --------------------------------------------------------------
c     First calculate relative humidity from Seinfeld (1986) p. 181.
c     The first  rh is the temperature dependent parameter a.
c     The second rh is the saturation vapor pressure of water.
c     The third  rh is the actual relative humidity as a fraction.
c     Then make sure rh is between 0 and 1.
c     --------------------------------------------------------------

      rh(:) = 1.0d0 - (373.15d0 / tempcol(:))

      rh(:) =
     &  1013.25d0 * Exp (13.3185d0 * rh(:)    -
     &                    1.9760d0 * rh(:)**2 -
     &                    0.6445d0 * rh(:)**3 -
     &                    0.1299d0 * rh(:)**4)

      rh(:) = watcol(:) * prescol(:) / rh(:)

      rh(:) = Max (Min (rh(:), 1.0d0), 0.0d0)


c     -----------------------------------------------------------------------
c     Now set up the constants to be used in the stratiform parameterization.
c     -----------------------------------------------------------------------

      where ((height_asl(:) < 15000.0d0) .and.
     &       (height_asl(:) >  5900.0d0))
        rhc(:) =  0.445d0
      end where
 
      where ((height_asl(:) <= 5900.0d0) .and.
     &       (height_asl(:) >  2350.0d0))
        rhc(:) =  0.715d0
      end where
 
      where  (height_asl(:) <= 2350.0d0)
        rhc(:) =  0.780d0
      end where


c     ----------------------------------------
c     Calculate the stratiform cloud fraction.
c     ----------------------------------------

      where (rh(:) > rhc(:) .and. 
     &        height_asl(:) < 15000.0d0)
 
        cfs(:) = 
     &    1.0d0- 
     &    Sqrt (1.0d0-(rh(:) - rhc(:)) /
     &               (1.0d0-rhc(:)))

      elsewhere

        cfs(:) = 0.0d0

      end where


c     --------------------------------------------------------------------
c     Now set up the constants to be used in the cumulus parameterization.
c     --------------------------------------------------------------------

      where ((height_asl(:) < 15000.0d0) .and.
     &       (height_asl(:) >  5900.0d0))
        c0(:) = 0.0131d0
        c1(:) = 0.0123d0
        c2(:) = 0.0030d0
      end where
 
      where ((height_asl(:) <= 5900.0d0) .and.
     &       (height_asl(:) >  2350.0d0))
        c0(:) = 0.0722d0
        c1(:) = 0.0760d0
        c2(:) = 0.0254d0
      end where
 
      where  (height_asl(:) <= 2350.0d0)
        c0(:) = 0.0337d0
        c1(:) = 0.0453d0
        c2(:) = 0.0237d0
      end where


c     -------------------------------------
c     Calculate the cumulus cloud fraction.
c     -------------------------------------

      where (cmfcol(:) > 0.01d0)

        cfc(:) = c0(:) +
     &           c1(:) *  Log (cmfcol(:)) +
     &           c2(:) * (Log (cmfcol(:)))**2

      elsewhere

        cfc(:) = 0.0d0

      end where


c           ---------------------------------------------------------
c           Compute total cloud fraction for combined large-scale and
c           convective clouds (GEOS-1 GCM descriptive documentation).
c           ---------------------------------------------------------

            clfrac(:) =
     &        1.0d0 - (1.0d0 - cfc(:)) * (1.0d0 - cfs(:))

       do ik =k1, k2

          if(moistqcol(ik) < -1.0d-5) then
            if (clfrac(ik) > 1.0d-2) then
c---------------------------Guangxing Lin -10-04-2011-----------
c----Note: Different from the calculation in Update_Sulfchem
c-----why the 3 lines below are not used here, while are in Update_Sulfchem???
c because the unit here is g/m^3 and in Update_Sulfchem is g/g
c                    lwccol(ik) =
c     &              lwccol(ik) * 1.0d-6 /
c     &              (const(ik,imgas_num) * MWTAIR / AVOGAD)
c---------------------------Guangxing Lin -10-04-2011-----------

cc                -----------------------------------------------------------
cc                Do not allow cloud in the lowest level according to NCAR
cc                CCM3 because LWC is prescribed as the maximum in the lowest
cc                level, Xhliu, 01/17/03.
cc                -----------------------------------------------------------

               if(ik == k1) lwccol(ik) = 0.0d0

cc                ------------------------------------------------------------
cc                Do not allow cloud liquid water when temperature below -25C.
cc                ------------------------------------------------------------

               if ( tempcol(ik) <248.0d0 ) lwccol(ik) = 0.0d0
    
           else
    
             lwccol(ik) = 0.0d0

           end if
 
        else
  
          lwccol(ik) = 0.0d0

        end if

      end do

cc      -------------------------
cc      Added by Xhliu, 02/05/03.
cc      -------------------------

        where ((clfrac(:) <= 1.0d-2) .or.
     &         (lwccol(:) <= 1.0d-9))
          clfrac(:) = 0.0d0
          lwccol(:) = 0.0d0
        end where

                
      return

      end



c ********************************************
c ** Begin of modifications by Luis Olcese, UM
c ** Changed to include secondary organic aerosols
      function ugm3(moleccm3, molw,density)
c---------------------gxlin------------------------------
c **** Convert a concentration expressed in mixing ratio
c **** to micrograms per cubic meter
c **** Molecular weight is needed
c-------------------------------gxlin-----------------
      real*8 :: ugm3, moleccm3, molw, density

      ugm3 = 1.d6 * 1.d6 / 6.02d23 * molw * moleccm3*density

      end
c ** End of modifications by Luis Olcese, UM
c ********************************************
