c=============================================================================
c
c $Id: rbhvrates.F,v 1.1 2003/05/16 22:21:52 akinorii Exp $
c
c CODE DEVELOPER
c   Original code from Sandy Sillman, U of M.
c   IMPACT quadchem modifications:  Akinori Ito.
c   2007 version for IMPACT:  Sandy Sillman
c
c FILE
c   rbhvrates.F
c
c ORIGINAL FILE
c   rbchemrates.f
c
c=============================================================================
c TEMPORARY DEBUG WRITES

c ----------------------------------------------------------------
c
c ROUTINE
c   rbhvrates

c DESCRIPTION
c   This routine combines the rate constants of the photolysis and thermal
c   processes.
c
c   It calculates photolysis rates from j-values (found from TUV Lookup)
c    and enters into the rate array (ratek)

c  NOTE:  shortened from original rbhvrates in rbchemrates.f
c   The original file determined ZENITH and other hv parameters,
c      called 'jvalpro' (TUV lookup)
c     and averaged for segments throughout the time interval.
c
c  In IMPACT, the TUV lookup is moved to TUV_lookup.F.
c  This shortened version uses j-values from TUV_lookup 
c   and assigns them to the ratek array.

c
c  Program  written by Sanford Sillman
c  ---------------------------------------------------------
c  Program history:
c    12/06 Initial program by Sandy Sillman based on boxchemv7.f 
c    7/07  IMPACT version
c  ---------------------------------------------------------
c     
c
      subroutine rbhvrates
     &                  (ratek,hvrate,c_dens,c_h2ogas)
c DEBUG
c    &                  (ratek,hvrate,c_dens,c_h2ogas, lhvdebug)

c
c SHORT VERSION: 
c Original calculates photolysis rates for specified location and time
c  based on interpolation in the lookup table TUVGRID2
c
c Average J-values are calculated over the specified time interval
c   Based on an looked-up j-values at specific times.
c
c Shortened version for IMPACT:  Takes J-values (from TUV_lookup.F)
c The J-values are entered into the rate constant array (RATEK)
c  for use in the chemistry solver.
c
c Photolysis reactions in the file REACTION.DAT
c  are identified by a RATE INDEX (NRK) from 1 to 56.
c  The index identifies the photolysis rate number in the lookup table.
c  The final j-value is the looked-up value multiplied by 
c   the coefficient RK1 (from REACTION.DAT)
c
c ----
c OLD INPUT:  moved to TUV_lookup:  See original in rbchemrates.f
c ----
c
c Input:  (from variables in common block)
c
c     
c Output: 
c    ratek() for photolysis reactions
c
c Called by:  Rate_Quad, from Update_Chem
c             (note imported j-values from TUV_Lookup from Qj_Update)
c
c History:
c  12/06 Written by Sandy Sillman from boxchemv7.f
c
c -------------------------------------------------------------
c
      use h_imp_control
      use h_imp_dims
      use h_rbchemmech
      use h_rbchemlocal
      IMPLICIT NONE 

c#     include "imp_control.h"
c#     include "imp_dims.h"
c#     include "rbchemmech.h"
c#     include "rbchemlocal.h"

c  VARIABLES FROM rbchemlocarr.h:  
c        (not allocated)
      double precision :: ratek(c_kvec,c_rdim)  ! Rate constants
c
c  LOCAL VARIABLES FOR J-VALUE CALCULATION
croy      double precision :: hvrate(c_kvec ,56) ! Interp hv over time interval
      double precision :: hvrate(c_kvec,num_qjs)
c                                            (now imported from TUV_lookup)
      double precision ::  c_h2ogas(c_kvec)  ! H2O abs humid molec/cm3
      double precision ::  c_dens(c_kvec)    ! density molec cm-3
      integer :: jc
c ---------------------------------------
      kk=1

c ENTER HV RATES FROM TUV TABLE INTO RATE CONSTANT ARRAY
c   INCLUDING MULTIPLICATIVE FACTOR AND SPECIAL HV RATE CONSTANTS

      do     j=1,c_nreac
        jc = c_nrk(j) 
croy        if(jc    .gt.0.and.jc    .lt.56) then
        if(jc.gt.0 .and. jc.lt.num_qjs) then
c vec                                    do kk=1,c_kvec   
         ratek( kk ,j) = hvrate( kk ,jc) 
     *     * c_rk(1,j)
c vec                                    enddo  !do kk=1,c_kvec   
        endif               !if(c_nrk(j).gt.0...

c  SPECIAL HV RATE CONSTANTS

c  RATE FOR 9.O3+hv=2OH (IF PRODUCT=OH)
        if(jc.eq.102) then
         if(c_nh2o.gt.0) then
c vec                                    do kk=1,c_kvec   
         ratek( kk ,j) = hvrate( kk , 2) 
     *     * c_rk(1,j)
     *     /( 1.+0.13181*c_dens( kk )/c_h2ogas(kk)    )
c    *     /( 1.+c_rk(2,j)*c_dens( kk )/c_h2ogas(kk)    )
c    *     /( 1.+0.13181*c_dens( kk )/xc( kk ,c_nh2o) )

c vec                                    enddo  !do kk=1,c_kvec   
         endif             !if(c_nh2o.gt.0) then
        endif            !if(jc.eq.102) then

c TEST WRITE
c       if(c_kkw.gt.0) then
c         if(c_rk(1,j) .eq. 1.000)  then
c           if(jc.eq.4.or.jc.eq.2.or.jc.eq.102) then
c             write(c_out,802) c_hour,  c_treac(1,j), ratek(c_kkw,j)
c 802           format(/,' TEST XHOUR, SPEC+HV JVAL= ',
c    *                  f6.2,2x,a8,2x,1pe10.3 )
          
c            endif
c          endif
c        endif

      enddo                !do     j=1,c_nreac


c  END HVRATES
2000  return
      end
c -----------------------------------------------------------


c ---------------------------------------------------------------
c --------------------------------------------------------------
c END chemrates SUBROUTINES
c --------------------------------------------------------------

c =============================================================
c
c
