
c=============================================================================
c
c $Id: synspc_update.F,v 1.5 2003/01/17 04:52:41 jrt Exp $
c
c CODE DEVELOPER
c   John Tannahill, LLNL
c   jrt@llnl.gov
c
c FILE
c   synspc_update.F
c
c ROUTINES
c   Update_Synspc
c   Calc_Synoz
c   Calc_Nodoz
c
c=============================================================================


c-----------------------------------------------------------------------------
c
c ROUTINE
c   Update_Synspc
c
c DESCRIPTION
c   This routine updates the synthetic species.
c
c ARGUMENTS
c   do_nodoz : do Nodoz?
c   latdeg   : latitude (deg)
c   press3e  : atmospheric pressure at the edge of each grid box (mb)
c   const    : species concentration, known at zone centers (mixing ratio)
c
c-----------------------------------------------------------------------------

      subroutine Update_Synspc
     &  (dttime, do_nodoz, latdeg, press3e, const)

      use h_imp_dims
      use h_imp_control, only: tdt
      implicit none

!#     include "imp_dims.h"


c     ----------------------
c     Argument declarations.
c     ----------------------

      logical :: do_nodoz
      real*8  :: latdeg (ju1_gl:j2_gl)
      real*8  :: press3e(ilo:ihi, julo:jhi, k1-1:k2)
      real*8  :: const  (i1:i2, ju1:j2, k1:k2, num_species)
      real*8  :: dttime !Gunagxing Lin 

      integer :: il, ij,ik,ic

      real*8 :: sum1

c     ----------------
c     Begin execution.
c     ----------------
c            sum1 = 0.0
c           do ik=k1, k2
c            do il=i1, i2
c              do ij=ju1, j2
c                do ic=1,num_species
c                 sum1              = sum1
c     &                                 + const(il, ij, ik, ic)
c                enddo   ! do ic=1,num_species
c               enddo
c              enddo
c            enddo
c          if(sum1 > 1.0*10.0**24) then  
c             write(6,*) "the begining of Init_Imp_Step -gxlin, sum1= ",
c     &                         sum1
c          end if

         tdt=dttime !Guangxing
c     ===============
      call Calc_Synoz
c     ===============
     &  (latdeg, press3e, const)


c            sum1 = 0.0
c           do ik=k1, k2
c            do il=i1, i2
c              do ij=ju1, j2
c                do ic=1,num_species
c                 sum1              = sum1
c     &                                 + const(il, ij, ik, ic)
c                enddo   ! do ic=1,num_species
c               enddo
c              enddo
c            enddo
c          if(sum1 > 1.0*10.0**24) then  
c                write(6,*) "after Calc_Synoz -gxlin, sum1= ",
c     &                         sum1
c           end if 
      if (do_nodoz) then
c-----Guangxing Lin for debug
c!c       ===============
c!        call Calc_Nodoz
c!c       ===============
c!     &    (latdeg, press3e, const)

      end if


          
      return

      end


c-----------------------------------------------------------------------------
c
c ROUTINE
c   Calc_Synoz
c
c DESCRIPTION
c   This routine calculates Synoz emissions (mixing ratio/s) and removes
c   Synoz from the surface.
c
c ARGUMENTS
c   latdeg  : latitude (deg)
c   press3e : atmospheric pressure at the edge of each grid box (mb)
c   const   : species concentration, known at zone centers (mixing ratio)
c
c-----------------------------------------------------------------------------

      subroutine Calc_Synoz
     &  (latdeg, press3e, const)

      use h_imp_phys_constants
      use h_imp_time_constants
      use h_imp_dims
      use h_imp_control
      implicit none

!#     include "imp_phys_constants.h"
!#     include "imp_time_constants.h"
!#     include "imp_dims.h"
!#     include "imp_control.h"


c     ----------------------
c     Argument declarations.
c     ----------------------

      real*8  :: latdeg (ju1_gl:j2_gl)
      real*8  :: press3e(ilo:ihi, julo:jhi, k1-1:k2)
      real*8  :: const  (i1:i2, ju1:j2, k1:k2, num_species)


c     -----------------------
c     Parameter declarations.
c     -----------------------

      integer, parameter ::
     &  KSTX_REM = 1,
     &  KEND_REM = 2

      real*8, parameter  ::
     &  LOLAT = -30.0d0,
     &  HILAT =  30.0d0

      real*8, parameter  ::
     &  LOPRS =  10.0d0,
     &  HIPRS =  70.0d0

      real*8, parameter  ::
     &  SYNOZ_RELAX_DAYS =   2.0d0,
     &  SYNOZ_SRCFLUX    = 475.0d0,  ! Synoz source flux (tg/yr)
     &  SYNOZ_SRFVAL     =   2.5d-8


c     ----------------------
c     Variable declarations.
c     ----------------------

      integer :: il, ij
      integer :: jsedge, jnedge
      integer :: kstx_emi, kend_emi

      logical, save :: doit     = .false.
      logical, save :: first    = .true.

      integer, save :: jstx_emi = -999
      integer, save :: jend_emi = -999

      real*8  :: rsecpdy, rsecpyr
      real*8  :: synoz_adjustment
      real*8  :: synoz_srcregion   ! relative size of Synoz source region to
                                   ! full atmosphere

      real*8, save :: synozadj_constant

      real*8  :: latdeg_edge(ju1_gl:j2_gl+1)  ! latitude at zone edge (deg)


c     ----------------
c     Begin execution.
c     ----------------

c     ==========
      if (first) then
c     ==========

        first = .false.

c       ---------------------------------------------------------
c       Determine latdeg_edge, but stay away from the Poles.
c       Synoz doesn't need latdeg_edge close to the Poles anyway.
c       latdeg_edge(ij) is the southern edge in latitude.
c       ---------------------------------------------------------

        latdeg_edge(ju1_gl)   = -90.0d0
        latdeg_edge(ju1_gl+1) = -90.0d0

        do ij = ju1_gl+2, j2_gl-1
          latdeg_edge(ij)     = (latdeg(ij-1) + latdeg(ij)) * 0.5d0
        end do

        latdeg_edge(j2_gl)    =  90.0d0
        latdeg_edge(j2_gl+1)  =  90.0d0


        if (Any ((latdeg_edge(ju1:j2)     >= LOLAT) .and.
     &           (latdeg_edge(ju1+1:j2+1) <= HILAT))) then

          doit = .true.

          jsedge   =
     &      Transfer
     &        (Minloc (latdeg_edge,
     &                 mask = (latdeg_edge >= LOLAT) .and.
     &                        (latdeg_edge <  HILAT)),
     &         1)

          jnedge   =
     &      Transfer
     &        (Maxloc (latdeg_edge,
     &                 mask = (latdeg_edge >  LOLAT) .and.
     &                        (latdeg_edge <= HILAT)),
     &         1)


          jstx_emi =
     &      Transfer
     &        (Minloc (latdeg_edge(ju1:j2),
     &                 mask = (latdeg_edge(ju1:j2)     >= LOLAT) .and.
     &                        (latdeg_edge(ju1+1:j2+1) <= HILAT)),
     &         1)

          jstx_emi = jstx_emi  + (ju1 - 1)


          jend_emi =
     &      Transfer
     &        (Maxloc (latdeg_edge(ju1:j2),
     &                 mask = (latdeg_edge(ju1:j2)     >= LOLAT) .and.
     &                        (latdeg_edge(ju1+1:j2+1) <= HILAT)),
     &         1)

          jend_emi = jend_emi + (ju1 - 1)


          synoz_srcregion =
     &      (Sin (latdeg_edge(jnedge) * RADPDEG) -
     &       Sin (latdeg_edge(jsedge) * RADPDEG)) *
     &      0.5d0 *
cc   &      ((HIPRS - LOPRS) / 993.407d0)
     &      ((HIPRS - LOPRS) / AVG_SRFPRS)


          rsecpyr = DBLE(SECPYR)

          synozadj_constant =
     &      tdt * SYNOZ_SRCFLUX *
     &      (MWTAIR / mw(isynoz_num)) /
     &      (rsecpyr * MASSATM * TGPKG * synoz_srcregion)

        end if

c     ======
      end if
c     ======


c     =========
      if (doit) then
c     =========

c       --------------------------
c       Calculate Synoz emissions.
c       --------------------------

        do ij = jstx_emi, jend_emi
          do il = i1, i2

            kstx_emi =
     &        Transfer
     &          (Maxloc (press3e(il,ij,k1:k2),
     &                   mask = (press3e(il,ij,0:k2-1) <= HIPRS) .and.
     &                          (press3e(il,ij,k1:k2)  >= LOPRS)),
     &           1)

            kstx_emi = kstx_emi + (k1 - 1)


            kend_emi =
     &        Transfer
     &          (Minloc (press3e(il,ij,k1:k2),
     &                   mask = (press3e(il,ij,0:k2-1) <= HIPRS) .and.
     &                          (press3e(il,ij,k1:k2)  >= LOPRS)),
     &           1)

            kend_emi = kend_emi + (k1 - 1)


            synoz_adjustment =
     &        synozadj_constant *
     &        ((HIPRS - LOPRS)  /
     &         (press3e(il,ij,kstx_emi-1) - press3e(il,ij,kend_emi)))


            const(il,ij,kstx_emi:kend_emi,isynoz_num) =
     &        const(il,ij,kstx_emi:kend_emi,isynoz_num) +
     &        synoz_adjustment


            if (io3_num /= 0) then
              const(il,ij,kstx_emi:kend_emi,io3_num) =
     &          const(il,ij,kstx_emi:kend_emi,io3_num) +
     &          synoz_adjustment
            end if

          end do
        end do

c     ======
      end if
c     ======


c     -----------------------------------
c     Calculate Synoz removal at surface.
c     -----------------------------------

      rsecpdy = DBLE(SECPDY)

      const(:,:,KSTX_REM:KEND_REM,isynoz_num) =
     &  SYNOZ_SRFVAL +
     &  Max ((const(:,:,KSTX_REM:KEND_REM,isynoz_num) - SYNOZ_SRFVAL),
     &       0.0d0) *
     &  Exp (-tdt / (SYNOZ_RELAX_DAYS * rsecpdy))


      return

      end


c-----------------------------------------------------------------------------
c
c ROUTINE
c   Calc_Nodoz
c
c DESCRIPTION
c   This routine calculates Nodoz emissions (mixing ratio/s) and
c   repartitions NOx and NOy above the troposphere.
c
c ARGUMENTS
c   latdeg  : latitude (deg)
c   press3e : atmospheric pressure at the edge of each grid box (mb)
c   const   : species concentration, known at zone centers (mixing ratio)
c
c-----------------------------------------------------------------------------

      subroutine Calc_Nodoz
     &  (latdeg, press3e, const)

      use h_imp_time_constants
      use h_imp_dims
      use h_imp_phys_constants
      use h_imp_control
      implicit none

!#     include "imp_phys_constants.h"
!#     include "imp_time_constants.h"
!#     include "imp_dims.h"
!#     include "imp_control.h"


c     ----------------------
c     Argument declarations.
c     ----------------------

      real*8  :: latdeg (ju1_gl:j2_gl)
      real*8  :: press3e(ilo:ihi, julo:jhi, k1-1:k2)
      real*8  :: const  (i1:i2, ju1:j2, k1:k2, num_species)


c     -----------------------
c     Parameter declarations.
c     -----------------------

      integer, parameter ::
     &  KEND_REM = 4,
     &  KSTX_REM = 1

      real*8, parameter ::
     &  HILAT =  30.0d0,
     &  LOLAT = -30.0d0

      real*8, parameter ::
     &  HIPRS  =  40.0d0,
     &  LOPRS  =   3.0d0,
     &  RLXPRS = 100.0d0

      real*8, parameter ::
     &  NODOZ_RELAX_DAYS = 3.0d0,
     &  NODOZ_SRCFLUX    = 2.295d0,  ! Nodoz as HNO3 (tg/yr)
     &  NODOZ_SRFVAL     = 0.5d-9,   
     &  RATIO            = 0.2d0,
     &  RATIO_RELAX_DAYS = 7.0d0


c     ----------------------
c     Variable declarations.
c     ----------------------

      logical, save :: doit  = .false.
      logical, save :: first = .true.

      integer :: il, ij, ic,ik
      real :: sum1
c     --------------------------------------------------------------------
c     jnedge : global index of the northernmost latitude edge that lies on
c              or is south of HILAT; used to define the meridional extent
c              of the region of emission
c     jsedge : global index of the southernmost latitude edge that lies on
c              or is north of LOLAT; used to define the meridional extent
c              of the region of emission
c     --------------------------------------------------------------------

      integer :: jnedge, jsedge

c     -----------------------------------------------------------------
c     kend_emi : index of the highest (lowest pressure)  cell that lies
c                entirely between HIPRS and LOPRS
c     kstx_emi : index of the lowest  (highest pressure) cell that lies
c                entirely between HIPRS and LOPRS
c     -----------------------------------------------------------------

      integer :: kend_emi, kstx_emi

c     ----------------------------------------------------------------
c     kend_rlx : index in const of the highest (lowest pressure)  cell
c                with an upper edge below (higher pressure) RLXPRS
c     kstx_rlx : index in const of the lowest  (highest pressure) cell
c                with a Synoz value above SYNOZ_TROP (?)
c     ----------------------------------------------------------------

      integer :: kend_rlx, kstx_rlx

c     ------------------------------------------------------------------
c     jend_emi : index of the northernmost cell in the emission region
c     jstx_emi : index of the southernmost cell on the current processor
c                that is located entirely north of LOLAT
c     ------------------------------------------------------------------

      integer, save :: jstx_emi = -999
      integer, save :: jend_emi = -999

      real*8  :: nodoz_increment

c     --------------------------------------------------------------
c     nodoz_srcregion : relative size of Nodoz source region to full
c                       atmosphere
c     --------------------------------------------------------------

      real*8  :: nodoz_srcregion

      real*8  :: rsecpdy, rsecpyr

      real*8, save  :: nodoz_incconstant

c     -----------------------------------------
c     latdeg_edge : latitude at zone edge (deg)
c     -----------------------------------------

      real*8  :: latdeg_edge(ju1_gl:j2_gl+1)

      real*8  :: nox_adj  (k1:k2)
      real*8  :: nox_ratio(k1:k2)

      real*8  :: noy_adj  (k1:k2)

      real*8  :: sum_nox  (k1:k2)
      real*8  :: sum_noy  (k1:k2)
      real*8  :: sumnoxadj, sumnoyadj

c     ----------------
c     Begin execution.
c     ----------------

c     ==========
      if (first) then
c     ==========

        first = .false.

c       ---------------------------------------------------------
c       Determine latdeg_edge, but stay away from the Poles.
c       Nodoz doesn't need latdeg_edge close to the Poles anyway.
c       latdeg_edge(ij) is the southern edge in latitude.
c       ---------------------------------------------------------

        latdeg_edge(ju1_gl)   = -90.0d0
        latdeg_edge(ju1_gl+1) = -90.0d0

        do ij = ju1_gl+2, j2_gl-1
          latdeg_edge(ij)     = (latdeg(ij-1) + latdeg(ij)) * 0.5d0
        end do

        latdeg_edge(j2_gl)    =  90.0d0
        latdeg_edge(j2_gl+1)  =  90.0d0


        if (Any ((latdeg_edge(ju1:j2)     >= LOLAT) .and.
     &           (latdeg_edge(ju1+1:j2+1) <= HILAT))) then

          doit = .true.

c         -----------------------------------------------------------------
c         The masks in the following two equations preselect only those
c         edges contained within LOLAT and HILAT.
c
c         Note that jsedge must be < jnedge for emission of Nodoz to occur.
c         -----------------------------------------------------------------

          jnedge =
     &      Transfer
     &        (Maxloc (latdeg_edge,
     &                 mask = (latdeg_edge >  LOLAT) .and.
     &                        (latdeg_edge <= HILAT)),
     &         1)

          jsedge =
     &      Transfer
     &        (Minloc (latdeg_edge,
     &                 mask = (latdeg_edge >= LOLAT) .and.
     &                        (latdeg_edge <  HILAT)),
     &         1)


c         -------------------------------------------------------------
c         The masks in the following two equations preselect only those
c         cells that are contained entirely within LOLAT and HILAT.
c         The result of the Minloc function is the smallest index,
c         counting from 1 in the subselected latdeg array, that
c         corresponds to a cell with a southern boundary lying on or
c         north of LOLAT and a northern boundary on or south of HILAT.
c         -------------------------------------------------------------

          jend_emi =
     &      Transfer
     &        (Maxloc (latdeg_edge(ju1:j2),
     &                 mask = (latdeg_edge(ju1:j2)     >= LOLAT) .and.
     &                        (latdeg_edge(ju1+1:j2+1) <= HILAT)),
     &         1)

          jstx_emi =
     &      Transfer
     &        (Minloc (latdeg_edge(ju1:j2),
     &                 mask = (latdeg_edge(ju1:j2)     >= LOLAT) .and.
     &                        (latdeg_edge(ju1+1:j2+1) <= HILAT)),
     &         1)

c         ------------------------------------------------------------
c         (ju1 - 1) below is the offset that must be added to jend_emi
c         and jstx_emi for indexing into the const array.
c         ------------------------------------------------------------

          jend_emi = jend_emi + (ju1 - 1)
          jstx_emi = jstx_emi + (ju1 - 1)


c         ------------------------------------------------------------
c         Note that AVG_SRFPRS below should be the true annual average
c         surface pressure for the global met field used.
c         ------------------------------------------------------------

          nodoz_srcregion =
     &      (Sin (latdeg_edge(jnedge) * RADPDEG) -
     &       Sin (latdeg_edge(jsedge) * RADPDEG)) *
     &      0.5d0 *
cc   &      ((HIPRS - LOPRS) / 993.407d0)
     &      ((HIPRS - LOPRS) / AVG_SRFPRS)


          rsecpyr = DBLE(SECPYR)

          nodoz_incconstant =
     &      tdt * NODOZ_SRCFLUX *
     &      (MWTAIR / mw(ihno3_num)) /
     &      (rsecpyr * MASSATM * TGPKG * nodoz_srcregion)

        end if

c     ======
      end if
c     ======


c     =========
      if (doit) then
c     =========

c       --------------------------
c       Calculate Nodoz emissions.
c       --------------------------

        do ij = jstx_emi, jend_emi
          do il = i1, i2

            kend_emi =
     &        Transfer
     &          (Minloc (press3e(il,ij,k1:k2),
     &                   mask = (press3e(il,ij,0:k2-1) <= HIPRS) .and.
     &                          (press3e(il,ij,k1:k2)  >= LOPRS)),
     &           1)

            kstx_emi =
     &        Transfer
     &          (Maxloc (press3e(il,ij,k1:k2),
     &                   mask = (press3e(il,ij,0:k2-1) <= HIPRS) .and.
     &                          (press3e(il,ij,k1:k2)  >= LOPRS)),
     &           1)

c           ----------------------------------------------------------
c           If, at some future time the parallel decomposition extends
c           to sending portions of the vertical column to different
c           processors, the k1 offset would need to be accounted for.
c           ----------------------------------------------------------

cc          kend_emi = kend_emi + (k1 - 1)
cc          kstx_emi = kstx_emi + (k1 - 1)


c           -----------------------------------------------------------
c           The high pressure edge index here (kstx_emi) is decremented
c           to account for the zero-based indexing of press3e.
c           -----------------------------------------------------------

            nodoz_increment =
     &        nodoz_incconstant *
     &        ((HIPRS - LOPRS)  /
     &         (press3e(il,ij,kstx_emi-1) - press3e(il,ij,kend_emi)))


c           -------------------------------
c           Actual Nodoz emission operator.
c           -------------------------------

            const(il,ij,kstx_emi:kend_emi,ihno3_num) =
     &        const(il,ij,kstx_emi:kend_emi,ihno3_num) +
     &        nodoz_increment


          end do
        end do

c            sum1 = 0.0
c           do ik=kstx_emi, kend_emi
c            do il=i1, i2
c              do ij = jstx_emi, jend_emi
c                do ic=1,num_species
c                 sum1              = sum1
c     &                                 + const(il, ij, ik, ic)
c                enddo   ! do ic=1,num_species
c               enddo
c              enddo
c            enddo
c            if (sum1 > 1.0*10.0**24) then
c                write(6,*) " Calc_Nodoz -gxlin---1, sum1= ",
c     &                         sum1
c            end if  
c     ======
      end if
c     ======

      if ((num_nox == 0) .and. (num_noy == 0)) then

c     -----------------------------------
c     Calculate Nodoz removal at surface.
c     -----------------------------------

      rsecpdy = DBLE(SECPDY)

      const(:,:,KSTX_REM:KEND_REM,ihno3_num) =
     &  NODOZ_SRFVAL +
     &  Max ((const(:,:,KSTX_REM:KEND_REM,ihno3_num) - NODOZ_SRFVAL),
     &       0.0d0) *
     &  Exp (-tdt / (NODOZ_RELAX_DAYS * rsecpdy))

c            sum1 = 0.0
c           do ik=KSTX_REM,KEND_REM 
c            do il=i1, i2
c              do ij=ju1, j2
c                do ic=1,num_species
c                 sum1              = sum1
c     &                                 + const(il, ij, ik, ic)
c                enddo   ! do ic=1,num_species
c               enddo
c              enddo
c            enddo
c            if (sum1 > 1.0*10.0**24) then
c                write(6,*) " Calc_Nodoz -gxlin---2, sum1= ",
c     &                         sum1
c            end if



      else

c     --------------------------
c     Calculate NOy partitioning.
c     --------------------------

c            sum1 = 0.0
c           do ik=k1, k2
c            do il=i1, i2
c              do ij=ju1, j2
c                do ic=1,num_nox
c                 sum1              = sum1
c     &                                 + const(il, ij, ik, nox_map(ic))
c                enddo   ! do ic=1,num_species
c               enddo
c              enddo
c            enddo
c            if (sum1 > 1.0*10.0**24) then
c                write(6,*) " Calc_Nodoz -gxlin---nox1, sum1= ",
c     &                         sum1
c            end if

c            sum1 = 0.0
c           do ik=kstx_rlx, k2
c           do ik=k1, k2
c            do il=i1, i2
c              do ij=ju1, j2
c                do ic=1,num_noy
c                 sum1              = sum1
c     &                                 + const(il, ij, ik, noy_map(ic))
c                enddo   ! do ic=1,num_species
c               enddo
c              enddo
c            enddo
c            if (sum1 > 1.0*10.0**24) then
c                write(6,*) " Calc_Nodoz -gxlin---noy1, sum1= ",

c     &                         sum1
c            end if
      do ij = ju1, j2
        do il = i1, i2

          kend_rlx =
     &      Transfer
     &        (Minloc (press3e(il,ij,k1:k2),
     &                 mask = (press3e(il,ij,k1:k2) >= RLXPRS)),
     &         1)

          kstx_rlx =
     &      Transfer
     &        (Maxloc (press3e(il,ij,k1:k2),
     &                 mask = (const(il,ij,k1:k2,isynoz_num) >
     &                         synoz_threshold)),
     &         1)
!         if(kstx_rlx == 0 ) then
!          write(6,*) 'gxlin, synoz,il,ij,maxloc', 
!    &       isynoz_num,synoz_threshold,il,ij,
!     &        Maxloc (press3e(il,ij,k1:k2),
!     &                 mask = (const(il,ij,k1:k2,isynoz_num) >
!     &                         synoz_threshold))
         
!           do ik=k1, k2
!           write(6,*) '(gxlin,press3e,ik)', press3e(il,ij,ik), 
!     &        const(il,ij,ik,isynoz_num),ik
!           end do
!         end if 

c         ----------------------------------------------------------
c         If, at some future time the parallel decomposition extends
c         to sending portions of the vertical column to different
c         processors, the k1 offset would need to be accounted for.
c         ----------------------------------------------------------

cc        kend_rlx = kend_rlx + (k1 - 1)
cc        kstx_rlx = kstx_rlx + (k1 - 1)


c         ------------------------------------
c         Assemble NOx and NOy sums and ratio.
c         ------------------------------------

          sum_nox(:) = 0.0d0

          do ic = 1, num_nox

            sum_nox = sum_nox +
     &                const(il,ij,:,nox_map(ic))

          end do

          sum_noy(:) = sum_nox(:)

          do ic = 1, num_noy

            sum_noy = sum_noy +
     &                const(il,ij,:,noy_map(ic))

          end do

          nox_ratio(:) = sum_nox(:) / sum_noy(:)
          
c          do ik=k1,k2
c             write(6,*) "gxlin,nox_ratio,sum_nox,sum_noy, il,ij,ik", nox_ratio(ik),sum_nox(ik),sum_noy(ik),il,ij,ik

c          end do          

c         ----------------------------------------------------------
c         Between the Synoz tropopause and RLXPRS, relax the NOx/NOy
c         ratio to the imposed RATIO with a RATIO_RELAX_DAYS
c         relaxation time constant.
c         ----------------------------------------------------------

          rsecpdy    = DBLE(SECPDY)

          nox_adj(:) = 1.0d0
         
c          if (kstx_rlx.eq.0.0) kstx_rlx=1.0
 
          if (kend_rlx >= kstx_rlx) then

            nox_adj(kstx_rlx:kend_rlx) =
     &        (RATIO +
     &         (nox_ratio(kstx_rlx:kend_rlx) - RATIO) *
     &         Exp(-tdt / (RATIO_RELAX_DAYS * rsecpdy))) /
     &        nox_ratio(kstx_rlx:kend_rlx)
       
c          do ik=kstx_rlx, kend_rlx
c          write(6,*) "gxlin, nox_ratio, nox_adj,tdt,rspecdy,", nox_ratio(ik),
c     &      nox_adj(ik),tdt,rsecpdy
c          write(6,*) "gxlin, RATIO, RATIO_RELAX_DAYS,il,ij,ik", RATIO,RATIO_RELAX_DAYS,
c     &    il,ij,ik

c          end do
c
          end if


c         --------------------------------------------------
c         Above RLXPRS (at lower pressures), force NOx / NOy
c         ratio, retaining subgroup partitioning.
c         --------------------------------------------------

          nox_adj(kend_rlx+1:k2) = RATIO / nox_ratio(kend_rlx+1:k2)
    
c          do ik=kend_rlx+1, k2
c           write(6,*) "gxlin, nox_adj, nox_ratio", nox_adj(ik), nox_ratio(ik)
c          write(6,*) "gxlin, RATIO, il,ij,ik", RATIO,
c     &    il,ij,ik
c          end do

c         ------------------
c         Adjust NOx values.
c         ------------------

          do ic = 1, num_nox

            const(il,ij,kstx_rlx:k2,nox_map(ic)) =
     &        nox_adj(kstx_rlx:k2) *
     &        const(il,ij,kstx_rlx:k2,nox_map(ic))

          end do


c         ------------------
c         Adjust NOy values.
c         ------------------

          noy_adj(:) = 1.0d0

          noy_adj(kstx_rlx:k2) =
     &      (1.0d0 - nox_adj  (kstx_rlx:k2) * nox_ratio(kstx_rlx:k2)) /
     &      (1.0d0 - nox_ratio(kstx_rlx:k2))

c          sumnoxadj =0.0
c          sumnoyadj = 0.0
c          do ik=k1, k2
c             sumnoxadj=sumnoxadj + nox_adj(ik)
c             sumnoyadj=sumnoyadj + noy_adj(ik)
c             end if
c          end do
c           write(6,*) "gxlin,sumnoxadj,sumnoyadj,il, ij", sumnoxadj,
c     &       sumnoyadj,il,ij 
          do ic = 1, num_noy

            const(il,ij,kstx_rlx:k2,noy_map(ic)) =
     &        noy_adj(kstx_rlx:k2) *
     &        const(il,ij,kstx_rlx:k2,noy_map(ic))

          end do

        end do
      end do

c            sum1 = 0.0
c           do ik=k1, k2
c            do il=i1, i2
c              do ij=ju1, j2
c                do ic=1,num_nox
c                 sum1              = sum1
c     &                                 + const(il, ij, ik, nox_map(ic))
c                enddo   ! do ic=1,num_species
c               enddo
c              enddo
c            enddo
cx            if(sum1 > 1.0*10.0**24) then
c                write(6,*) " Calc_Nodoz -gxlin---nox2, sum1=  ",
c     &                         sum1, "num_nox= ", num_nox, "kend_rlx= ", kend_rlx
c            end if    

c            sum1 = 0.0
c           do ik=k1, k2
c            do il=i1, i2
c              do ij=ju1, j2
c                do ic=1,num_noy
c                 sum1              = sum1
c     &                                 + const(il, ij, ik, noy_map(ic))
c                enddo   ! do ic=1,num_species
c               enddo
c              enddo
c            enddo
c            if(sum1 > 1.0*10.0**24) then
c                write(6,*) " Calc_Nodoz -gxlin---noy2, sum1= ",

c     &                         sum1,"num_noy= ", num_noy, "kstx_rlx= ", kstx_rlx
c            end if    
      end if


      return

      end

